import Highcharts from 'highcharts';
import HighchartsExport from 'highcharts/modules/exporting';
import Offline from 'highcharts/modules/offline-exporting';
import HighchartsHeatmap from 'highcharts/modules/heatmap';
import HighchartsTreemap from 'highcharts/modules/treemap';
import HighchartsMore from 'highcharts/highcharts-more';
import HighchartsPie from 'highcharts/modules/variable-pie.js';
import Annotations from 'highcharts/modules/annotations.js';
import Solid from 'highcharts/modules/solid-gauge.js';
import exportData from 'highcharts/modules/export-data'
import Zh from './zb';

HighchartsExport(Highcharts);
Offline(Highcharts);
exportData(Highcharts);
HighchartsMore(Highcharts);
HighchartsPie(Highcharts);
HighchartsHeatmap(Highcharts);
HighchartsTreemap(Highcharts);
Annotations(Highcharts);
Solid(Highcharts);
Zh(Highcharts);
Highcharts.wrap(Highcharts.Tooltip.prototype, 'getLabel', function (proceed) {
  let isCreate = !this.label;
  let label = proceed.apply(this, Array.prototype.slice.call(arguments, 1));
  if (isCreate) {
    let chart = this.chart;
    this.clickEvent = Highcharts.addEvent(label.div || label.element, 'click', function (e) {
      //alert('tooltip点击');
      // console.log(chart.hoverPoint)
      chart.userOptions.chart.events.click.call(chart, chart.hoverPoint, 'tooltip');
      e.cancelBubble = true;
      e.stopPropagation();
      e.preventDefault();
      e.stopImmediatePropagation();
      return false;
    });
  }
  return label;
});
function setOpt() {
  var obj;

  return function () {
    if (obj) {
      return obj
    } else {
      // 获取主题配置
      let opt = sessionStorage.getItem('chartObj');

      if (!opt) {
        return {}
      }
      opt = JSON.parse(opt);

      let {
        ft1,
        ft2,
        cl1,
        cl2,
        ff
      } = opt;

      obj = {
        lang: {
          thousandsSep: ','
        },
        chart: {
          style: {
            fontFamily: ff,
            fontSize: ft1,
            color: cl1
          }
        },
        title: {
          style: {
            fontFamily: ff,
            fontSize: ft2,
            color: cl1
          }
        },
        subtitle: {
          style: {
            fontFamily: ff,
            fontSize: ft1,
            color: cl2
          }
        },
        xAxis: {
          labels: {
            style: {
              fontFamily: ff,
              fontSize: ft1,
              color: cl2
            }
          }
        },
        yAxis: {
          labels: {
            style: {
              fontFamily: ff,
              fontSize: ft1,
              color: cl2
            }
          }
        },
        zAxis: {
          labels: {
            style: {
              fontFamily: ff,
              fontSize: ft1,
              color: cl2
            }
          }
        },
        tooltip: {
          headerFormat: '<span >{point.key}</span><br/>',
          style: {
            fontFamily: ff,
            fontSize: ft2,
            color: cl1
          }
        },
        labels: {
          style: {
            fontFamily: ff,
            fontSize: ft2,
            color: cl2
          }
        },

        legend: {
          itemStyle: {
            fontFamily: ff,
            fontSize: ft2,
            color: cl1
          }
        },
        plotOptions: {
          series: { // 针对所有数据列有效的配置
            dataLabels: {
              style: {
                fontFamily: ff,
                fontSize: ft2,
                color: cl2
              }
            }
          }
        }
      }

      return obj
    }
  }
}

let _opt = setOpt()();

Highcharts.setOptions(
  _opt,
  global: { useUTC: true }
);

export default Highcharts;
