import { Component } from 'react';
import styles from './style.scss';
import highchart from './highchart';
import EasyChart from './EasyChart';

class ChartItem extends Component {
  state = {
    tempChart: null,
    option: null
  }

  componentDidMount() {
    const {
      loadOk,
      option,
      PointMouseOver,
      PointMouseOut,
      selectActive,
      callBack
    } = this.props;
    if (!option)
      return;

    // 检测是否需要添加联动事件
    if (PointMouseOver && PointMouseOut && selectActive) {
      option.plotOptions = {
        ...option.plotOptions,
        series: {
          point: {
            events: {
              mouseOver: function () {
                PointMouseOver(this.category)
              },
              mouseOut: function () {
                PointMouseOut(this.category)
              }
            }
          }
        }
      }
    }

    let tempChart = highchart.chart(this.chartBox, option, callBack);
    this.setState({ tempChart, option: option });
    highchart.addEvent(tempChart.container, 'click', e => {
      if (document.activeElement.tagName === 'INPUT') {
        document.activeElement.blur();
      }
    })
    loadOk(tempChart);
  }

  componentWillReceiveProps(newProps) {
    const {
      loadOk,
      option,
      PointMouseOver,
      PointMouseOut,
      selectActive,
      callBack
    } = newProps;
    if (!option || !this.state.tempChart)
      return;

    let { option: nowOption, tempChart: oldChart } = this.state;

    // 如果没有变化不进行渲染
    if (JSON.stringify(nowOption) == JSON.stringify(option) || oldChart == null) {
      return
    }

    // 检测是否需要添加联动事件
    if (PointMouseOver && PointMouseOut && selectActive) {
      option.plotOptions = {
        ...option.plotOptions,
        series: {
          point: {
            events: {
              mouseOver: function () {
                PointMouseOver(this.category)
              },
              mouseOut: function () {
                PointMouseOut(this.category)
              }
            }
          }
        }
      }
    }

    let tempChart = highchart.chart(this.chartBox, option, callBack);
    this.setState({ tempChart, option: option })
    loadOk(tempChart);
  }

  // 组件卸载后销毁图表
  componentWillUnmount() {
    this.setState = null;
    const { option } = this.props;
    const { tempChart } = this.state
    if (option && tempChart != null) {
      // 销毁图表
      this.state.tempChart.destroy()
    }
  }

  componentDidUpdate() {
    if (this.state.tempChart) {
      this.state.tempChart.reflow()
    }
  }

  render() {
    const { option, style } = this.props;
    if (!option) {
      return (<div className={styles.noData}>
        <p>没有数据</p>
      </div>)
    } else {
      return (
        <div
          style={{ ...style }}
          ref={(chart) => { this.chartBox = chart }}>
        </div>
      )
    }
  }
}

ChartItem.EasyChart = EasyChart;

export default ChartItem
