import React from "react";
import DownLoad from "_c/DownLoad";
import Select from "_c/Select";
import DropDownMul from "_c/DropDownMul/index";
import DorpMulItem from "_c/DropDownMul/MulItem";
import Option from "_c/Select/Option";
import { Radio, Input } from "antd";
import styles from "./index.scss";
const { Search } = Input;

const getTypes = ({ types, typeValue, handelType, handelTypes }) => {
  if (types === undefined) {
    return;
  }
  if (types && types[0] && types[0].title) {
    // typeValue
    let dft = types[0].title;
    if (typeValue && types[0]) {
      typeValue.forEach((ty) => {
        if (ty.key === types[0].key) {
          dft = ty.title;
        }
      });
    }

    return (
      <Select
        style={{ float: "right", marginRight: 8, width: 130 }}
        size={"small"}
        defaultFirst={true}
        value={dft}
        onChange={handelType}
      >
        {types.map((item) => {
          return (
            <Option
              data-click={{
                type: `${item.key}-${item.value}`,
                title: `${item.key}`,
              }}
              object={item}
              key={`${item.key}_${item.value}`}
              value={item.title}
            >
              {item.title}
            </Option>
          );
        })}
      </Select>
    );
  } else {
    // typeValue
    return types.map((titem, i) => {
      let dft = titem[0].title;
      if (typeValue && types[0]) {
        typeValue.forEach((ty) => {
          if (ty.key === titem[0].key) {
            dft = ty.title;
          }
        });
      }
      return (
        <Select
          style={{ float: "right", marginRight: 8, width: 130 }}
          size={"small"}
          defaultFirst={true}
          key={`${dft}_${i}`}
          value={dft}
          onChange={(val) => handelTypes(val, i)}
        >
          {titem.map((item) => {
            return (
              <Option
                data-click={{
                  type: `${item.key}-${item.value}`,
                  title: `${item.key}`,
                }}
                key={`${item.key}_${item.value}`}
                object={item}
                value={item.title}
              >
                {item.title}
              </Option>
            );
          })}
        </Select>
      );
    });
  }
};
//filters [{},{}], {title: 'ff1', value: 1, key: 'ff1'}
const getFilters = ({ filters, filterValue, handelFilter }) => {
  if (filters && !filters.props) {
    return filters.map((fitem, i) => {
      let val = fitem[0].title;
      const valKey = fitem[0].key;
      if (filterValue) {
        Object.keys(filterValue).forEach((key) => {
          val = key === valKey ? filterValue[key].title : val;
        });
      }
      return (
        <Select
          style={{ float: "right", marginRight: 8, width: 130 }}
          size={"small"}
          value={val}
          onChange={handelFilter}
        >
          {fitem.map((item) => {
            return (
              <Option
                data-click={{
                  type: `${item.key}-${item.value}`,
                  title: `${item.key}`,
                }}
                object={item}
                value={item.title}
              >
                {item.title}
              </Option>
            );
          })}
        </Select>
      );
    });
  }
  return filters;
};
const getTags = ({ RadioChange, filterValue, tags }) => {
  return (
    <Radio.Group
      defaultValue="a"
      buttonStyle="solid"
      onChange={RadioChange}
      defaultValue={filterValue.columnType.value}
      style={{ marginBottom: 8 }}
    >
      {tags.map((item) => {
        return (
          <Radio.Button key={item.value} value={item.value}>
            {item.title}
          </Radio.Button>
        );
      })}
    </Radio.Group>
  );
};

const getMulFilter = ({ mulFilter, handelFilter }) => {
  return (
    <DropDownMul
      title="选择栏目"
      style={{ float: "right", marginRight: 8 }}
      onChange={(val) => handelFilter({ value: val, key: mulFilter[0].key })}
    >
      {mulFilter.map((mitem) => {
        return <DorpMulItem value={mitem.value}>{mitem.title}</DorpMulItem>;
      })}
    </DropDownMul>
  );
};

const Func = (props) => {
  const {
    downTypes,
    conSearch,
    filters,
    filter,
    types,
    tags,
    pmark,
    downChart,
    downParam,
    searchChange,
    RadioChange,
    filterValue,
    handelFilter,
    handelType,
    handelTypes,
    typeValue,
    mulFilter,
    chartDom,
  } = props;

  const hscon =
    downTypes || conSearch || filters || filter || types || tags || mulFilter;
  if (!hscon) return null;
  return (
    <div
      className={styles["chart_s_con"]}
      style={tags ? { marginTop: 16 } : {}}
    >
      {tags && getTags({ RadioChange, filterValue, tags })}
      {downTypes && (
        <DownLoad
          {...downParam}
          types={downTypes}
          pmark={pmark}
          // chart={this.state.chart !== null ? this.state.chart : this.chartDom}>
          chart={downChart}
          chartDom={chartDom}
        ></DownLoad>
      )}
      {conSearch && (
        <Search
          placeholder="全标题搜索"
          onSearch={searchChange}
          size="small"
          style={{ width: 200, float: "right" }}
        />
      )}
      {mulFilter && getMulFilter({ mulFilter, handelFilter })}
      {filters && getFilters({ filters, filterValue, handelFilter })}
      {filter && filter}
      {types &&
        types !== undefined &&
        getTypes({ types, typeValue, handelType, handelTypes })}
    </div>
  );
};

export default Func;
