import styles from './styles.scss';
import PropTypes from 'prop-types'
import Icon from '../Icon';
import Plus from './plus';
import format from './format';
import Value from './Value';
import update from 'immutability-helper'
import _event from '../event';
import moment from 'moment';

// 获取日期类型
const getDateType = (value) => {
  switch (value) {
    case 'year':
      return 1;
    case 'quarter':
      return 2;
    case 'month':
      return 3;
    case 'week':
      return 4;
    case 'day':
      return 5;
    case 'halfYear':
      return 6;
  }
}

//日期选择类型
const byMonth = localStorage.getItem("lang") == 'en' ? 'By Month' : '按月查看'; // uvnb国际化临时采用这种方式，后续需要使用react intl
const dType = {
  day: {
    value: 'day',
    name: '按日查看'
  },
  week: {
    value: 'week',
    name: '按周查看'
  },
  month: {
    value: 'month',
    name: byMonth
  },
  quarter: {
    value: 'quarter',
    name: '按季查看'
  },
  year: {
    value: 'year',
    name: '按年查看'
  },
  halfYear: {
    value: 'halfYear',
    name: '按半年查看'
  }
};

const getTimes = (value) => {
  let flag = value.includes('Q')
  if (value.includes('Q')) {
    let temp = value.split('Q');
    let [year, month] = temp;
    year = parseInt(year)
    month = parseInt(month) * 3 - 1;
    let tempDate = new Date(year, month + 1, 0).getTime()
    return tempDate;
  } else if (value.includes('-')) {
    let temp = value.split('-');
    let [year, month, day] = temp
    year = parseInt(year)
    month = parseInt(month) - 1;
    day = day
      ? day
      : 1;
    day = parseInt(day);

    let tempDate = new Date(year, month, day).getTime()
    return tempDate
  } else {
    let tempDate = new Date(parseInt(value) + 1, 0, 0).getTime()
    return tempDate
  }

}

class Dates extends React.Component {
  constructor(props) {
    super(props);
    let defaultValue = this.getDefalutDate(this.props)
    let { selectType, dType } = this.props

    selectType = dType
      ? dType
      : selectType[0]

    this.state = {
      value: defaultValue, // 用于显示的时间格式
      drop: false, // 展开下拉
      dateType: selectType, // 日期选择类型
      active: false, //全局关闭是否其作用
      weekSelect: []
    }
  }

  // 销毁事件
  componentWillUnmount() {
    _event.removeEvent(document, 'click', this.hideDrop);
  }

  componentWillReceiveProps(nextProps) {
    let { dType } = nextProps;
    let temp = this.state.dateType;

    if (dType !== this.props.dType && dType) {
      temp = dType;
    }
    this.setState(() => {
      let defaultValue = this.getDefalutDate(nextProps)
      return {
        value: defaultValue,
        dateType: temp,
      }
    })
  }

  enAble = (e) => {
    e.nativeEvent.stopImmediatePropagation();
  }

  getDefalutDate = (props) => {
    let { defaultValue } = props;
    // 判断是否选择了值
    return defaultValue
  }

  // 单选状态下的选择
  radioSelect = (value, discript) => {
    this.setState({
      value: value,
      drop: false,
      active: false
    }, () => {
      let dataType = getDateType(this.state.dateType);
      this.props.onChange(value, dataType, discript);
      _event.removeEvent(document, 'click', this.hideDrop);
    })
  }

  // 多选时选择
  multiSelect = (value) => {
    let temp = this.state.value;

    // 判断是否为字符串，如果是就转换为数组
    if (typeof temp == 'string') {
      temp = []
    }

    const { size, jumpYear } = this.props;
    const { dateType, value: OldValue } = this.state;
    if (jumpYear && dateType === 'month') {
      let endDate = moment(value).format('YYYY');
      let flag = OldValue.every(item => {
        return item.includes(endDate)
      })
      if (!flag) {
        this.setState({ value: [value] })
        return
      }
    }
    // 判断是否大于长度
    if (temp.length > size) {
      return;
    } else if (temp.length == size) {
      if (temp.includes(value)) {
        temp = temp.filter(item => item !== value)
        this.setState({ value: temp })
      } else {
        return;
      }
    } else {
      if (temp.includes(value)) {
        temp = temp.filter(item => item !== value)
      } else {
        temp.push(value)
      }
      this.setState({ value: temp })
    }
  }

  // 区域选择
  sectionSelect = (value) => {
    let temp = this.state.value;
    if (temp.length > 1) {
      let before = getTimes(temp);
      let after = getTimes(value);
      if (before < after) {
        temp = `${temp} 至 ${value}`
      } else {
        temp = `${value} 至 ${temp}`
      }
    } else {
      temp = value
    }

    let active = temp.split('至');
    if (active.length > 2) {
      this.setState((prevState) => {
        return { value: value }
      })
    } else {
      this.setState({
        value: temp,
        drop: false,
        active: false
      }, () => {
        let dataType = getDateType(this.state.dateType)
        this.props.onChange(temp, dataType, null);
        _event.removeEvent(document, 'click', this.hideDrop);
      })
    }
  }

  // 区域选择
  sectionWeekSelect = (value) => {
    const { weekSelect } = this.state;

    let startTime;
    let endTime;
    if (weekSelect.length == 1) {
      startTime = weekSelect[0].split('至')[0];
      endTime = value.split('至')[1]

      let before = getTimes(startTime);
      let after = getTimes(endTime);
      let temp;
      if (before < after) {
        temp = `${startTime} 至 ${endTime}`
      } else {
        endTime = value.split('至')[0];
        startTime = weekSelect[0].split('至')[1];
        temp = `${endTime} 至 ${startTime}`
      }
      let arr = [
        ...weekSelect,
        value
      ]
      this.setState({
        value: temp,
        drop: false,
        active: false,
        weekSelect: arr
      }, () => {
        let dataType = getDateType(this.state.dateType)
        this.props.onChange(temp, dataType, null);
        _event.removeEvent(document, 'click', this.hideDrop);
      })
    } else {
      this.setState({ value: value, weekSelect: [value] })
    }
  }

  // 多选时清空当前标签
  close = async (index, e) => {
    e.stopPropagation()
    e.nativeEvent.stopImmediatePropagation();
    await this.setState((prevState) => {
      let temp = update(prevState, {
        value: {
          $splice: [
            [index, 1]
          ]
        }
      })
      return {
        ...temp
      }
    })
    let dataType = getDateType(this.state.dateType)
    this.props.onChange(this.state.value, dataType, null)
  }

  // 显示下拉框
  showDrop = (e) => {
    const { onChange } = this.props;
    const { active, drop, value, dateType } = this.state;

    if (drop) {
      this.setState({ active: false, drop: false })
      let temp = getDateType(dateType)
      onChange(value, temp, null)
      _event.removeEvent(document, 'click', this.hideDrop);
    } else {
      this.setState({ active: true, drop: true })
      _event.addEvent(document, 'click', this.hideDrop);
    }
  }

  // 隐藏下拉框
  hideDrop = () => {
    this.setState({
      drop: false,
      active: false
    }, () => {
      let dataType = getDateType(this.state.dateType)
      this.props.onChange(this.state.value, dataType, null)
      _event.removeEvent(document, 'click', this.hideDrop);
    })
  }

  render() {
    let {
      placehold,
      select,
      selectType,
      defaultValue,
      min,
      max,
      size,
      typeChage,
      describe,
      sectionNum,
      inputStyle,
      direction,
      getword
    } = this.props;

    const { value, dateType, drop, weekSelect } = this.state;
    selectType = selectType.map(item => {
      return dType[item]
    })

    min = format.getTime(min);
    max = format.getTime(max);

    // 根据单选还是多选定义宽度
    let w = 'auto';
    //判断用户是否传入了描述文本
    placehold = placehold
      ? placehold
      : '请选择日期';

    const showValue = () => {
      if (value !== null || value !== '') {
        return (
          <Value
            inputStyle={inputStyle}
            size={size}
            dateType={dateType}
            select={select}
            value={value}
            close={this.close}
            describe={describe}
            getword={getword}
          />
        )
      } else {
        return <span>{placehold}</span>
      }
    }

    // 是否显示日历展开下拉
    const showDrop = () => {
      let fx = direction
        ? direction
        : 'left';
      if (fx == 'left') {
        fx = styles.drop
      } else {
        fx = styles.drop_1
      }
      if (drop) {

        if (!value) null;
        return (<div className={fx} onClick={this.enAble}>
          {/* 左侧切换日期类型菜单 */}
          <div className={styles.left}>
            {
              selectType.map((item, i) => {
                let cls = dateType === item.value
                  ? styles.active
                  : "";
                let _act = dateType === item.value
                  ? 'car-bg-three__vi'
                  : ''

                return (
                  <p
                    key={i}
                    className={`${cls} ${_act}  car-bg-secend-hover__vi`}
                    onClick={() => {
                      if (dateType === item.value) return;
                      if (typeChage) {
                        typeChage(item.value)
                      }
                      setTimeout(() => {
                        this.setState(() => {
                          return { dateType: item.value }
                        })
                      }, 0)
                    }}
                  >
                    {getword ? getword(item.name) : item.name}
                  </p>)
              })
            }
          </div>
          {/* 切换日期类型 */}
          <Plus
            sectionSelect={this.sectionSelect}
            multiSelect={this.multiSelect}
            weekSelect={weekSelect}
            sectionWeekSelect={this.sectionWeekSelect} radioSelect={this.radioSelect}
            defaultValue={value}
            select={select}
            dateType={dateType}
            sectionNum={sectionNum}
            min={min}
            max={max}
            getword={getword}
          />
        </div>)
      }

      return null
    }

    return (
      <div className={styles['car-date-input']}>
        {
          this.props.disabled &&
          <div className={styles['car-date-disable']}></div>
        }
        <div className={styles.input}>
          <div
            onClick={this.showDrop}
            className={`${styles.discript} car-border-hover__vi`}
            style={{
              width: w,
              ...inputStyle
            }}>
            {showValue()}
            <Icon type={'rili1'} />
          </div>
          {showDrop()}
        </div>
      </div>
    )
  }
}

Dates.propTypes = {
  placehold: PropTypes.string, //日期插件描述
  select: PropTypes.string, // 多选还是单选  radio：单选  multi：多选
  min: PropTypes.string, // 最小值
  max: PropTypes.string, //最大值
  onChange: PropTypes.func, // 回掉函数
}

export default Dates;
