import styles from "../styles.scss";
import Icon from "../../Icon";
import { Component } from "react";

const getTimes = (value) => {
  let temp = value.split("-");
  let [year, month, day] = temp;

  year = parseInt(year);
  month = parseInt(month) - 1;
  day = day ? day : 1;
  day = parseInt(day);

  let tempDate = new Date(year, month, day).getTime();
  return tempDate;
};

const getTimsMax = (value, len = 0, max) => {
  let temp = value.split("-");
  let [y, m, d] = temp;
  y = parseInt(y);
  m = parseInt(m);
  d = parseInt(d);

  // 获取本月多少天
  let benM = new Date(y, m, 0).getDate();
  d += len;

  if (d > benM) {
    d -= benM;
    m++;

    if (m > 12) {
      m = 1;
      y++;
    }
  }

  let tempTime = new Date(y, m - 1, d).getTime();
  tempTime = Math.min(tempTime, max);
  return tempTime;
};
const getTimsMin = (value, len = 0, min) => {
  let temp = value.split("-");
  let [y, m, d] = temp;
  y = parseInt(y);
  m = parseInt(m);
  d = parseInt(d);

  // 获取上个月多少天
  let benM = new Date(y, m - 1, 0).getDate();
  d -= len;

  if (d < 0) {
    d += benM;
    m--;

    if (m < 1) {
      m = 12;
      y--;
    }
  }

  let tempTime = new Date(y, m - 1, d).getTime();
  tempTime = Math.max(tempTime, min);
  return tempTime;
};

class Month extends Component {
  constructor(props) {
    super(props);

    this.state = {
      year: "",
      month: "",
      day: {
        before: [], //上个月日期
        current: [], // 当前月日期
        after: [], // 下个月日期
      },
    };
  }

  componentDidMount() {
    const { defaultValue } = this.props;
    let m;
    let y;

    if (defaultValue) {
      let temp = defaultValue.split("至")[0];
      let [y1, m1] = temp.split("-");
      y = parseInt(y1);
      m = parseInt(m1);
    } else {
      let dt = new Date();
      y = dt.getFullYear();
      m = dt.getMonth() + 1;
    }
    m = m <= 9 ? `0${m}` : m;
    this.getDays(m, y);
  }

  // 年份增加
  addYear = async () => {
    let { month, year } = this.state;
    year = parseInt(year);
    year++;
    this.getDays(month, year);
  };

  // 年份减少
  reduceYear = async () => {
    let { month, year } = this.state;
    year = parseInt(year);
    year--;
    this.getDays(month, year);
  };

  // 增加月份
  addMonth = async (value) => {
    let { month, year } = this.state;
    month = parseInt(month);
    if (month >= 12) {
      month = 0;
      year = parseInt(year) + 1;
    }
    month++;
    month = month <= 9 ? `0${month}` : month;
    this.getDays(month, year);
  };

  // 减少月份
  reduceMonth = async () => {
    let { month, year } = this.state;
    month = parseInt(month);
    if (month <= 1) {
      month = 13;
      year = parseInt(year) - 1;
    }
    month--;
    month = month <= 9 ? `0${month}` : month;
    this.getDays(month, year);
  };

  // 获取日期
  getDays = (m, y) => {
    let month = m;
    let year = y;
    month = +month;
    year = +year;

    // 获取当前月 的 1 号是星期几
    let tempDate = new Date(year, month - 1, 1);
    tempDate = tempDate.getDay();
    tempDate = tempDate == 0 ? 7 : tempDate;

    let beforeList = []; // 上个月数组
    let currentList = []; // 当月数组
    let afterist = []; // 下个月数组

    // 获取上个月日期
    if (tempDate != 1) {
      // 设置为上个月最后一天
      let beforeDate = new Date(year, month - 1, 0);
      beforeDate = beforeDate.getDate();
      for (let i = 0; i < tempDate - 1; i++) {
        beforeList.push(beforeDate - i);
      }
      beforeList.reverse();
    }

    // 获取当月的日期
    let nowDate = new Date(year, month, 0);
    nowDate = nowDate.getDate();
    for (let i = 0; i < nowDate; i++) {
      currentList.push(nowDate - i);
    }
    currentList.reverse();

    // 获取下个月日期
    let len = 42 - currentList.length - beforeList.length;
    for (let i = 1; i < len + 1; i++) {
      afterist.push(i);
    }

    this.setState(() => {
      let temp = {
        before: beforeList, //上个月日期
        current: currentList, // 当前月日期
        after: afterist, // 下个月日期
      };
      return { day: temp, year: y, month: m };
    });
  };

  // 返回国际化
  getWords = (value) => {
    const { getword } = this.props;

    if (getword) {
      let reg = /年/g;

      if (reg.test(value)) {
        let str = value.replace(reg, "");
        return str;
      }

      return getword(value);
    }
    return value;
  };

  render() {
    const {
      sectionSelect,
      min,
      max,
      defaultValue,
      sectionNum,
    } = this.props;
    const { year, month, day } = this.state;
    const { after, before, current } = day;

    let temp = defaultValue.split("至");

    let beginTimes;
    let endTimes;

    let tempMin = min;
    let tempMax = max;
    if (temp.length > 1) {
      beginTimes = getTimes(temp[0]);
      endTimes = getTimes(temp[1]);
    } else {
      if (sectionNum && temp[0] != "") {
        tempMax = getTimsMax(temp[0], sectionNum, max);
        tempMin = getTimsMin(temp[0], sectionNum, min);
      }
    }

    return (
      <div className={styles.right}>
        <div className={styles.top}>
          <Icon
            type={"zuofanye1"}
            clm={"car-color-hover__vi"}
            onClick={this.reduceYear}
          />
          <Icon
            type={"zuofanye"}
            clm={"car-color-hover__vi"}
            onClick={this.reduceMonth}
          />
          <div className={styles.selectDate}>
            <span> {this.getWords(`${year}年`)}</span>
            <span>{this.getWords(`${month}月`)}</span>
          </div>
          <Icon
            type={"youfanye1"}
            clm={"car-color-hover__vi"}
            onClick={this.addMonth}
          />
          <Icon
            type={"youfanye"}
            clm={"car-color-hover__vi"}
            onClick={this.addYear}
          />
        </div>
        <div className={styles.conterDay}>
          <em>{this.getWords("一")}</em>
          <em>{this.getWords("二")}</em>
          <em>{this.getWords("三")}</em>
          <em>{this.getWords("四")}</em>
          <em>{this.getWords("五")}</em>
          <em>{this.getWords("六")}</em>
          <em>{this.getWords("日")}</em>
          {before.map((item, i) => {
            return (
              <span key={i} className={styles.hui}>
                {item}
              </span>
            );
          })}
          {current.map((item, i) => {
            let nowY = parseInt(year);
            let nowM = parseInt(month);
            let dt = new Date(nowY, nowM - 1, item);
            let tempTiem = dt.getTime();

            nowM = nowM <= 9 ? `0${nowM}` : nowM;

            let day = item <= 9 ? `0${item}` : item;

            if (tempTiem > tempMin && tempTiem < tempMax) {
              if (beginTimes && endTimes) {
                if (defaultValue.includes(`${nowY}-${nowM}-${day}`)) {
                  return (
                    <span
                      key={i}
                      onClick={sectionSelect.bind(
                        this,
                        `${nowY}-${nowM}-${day}`
                      )}
                      className={`${styles.selected} car-bg__vi`}
                    >
                      {item}
                    </span>
                  );
                } else {
                  if (tempTiem > beginTimes && tempTiem < endTimes) {
                    return (
                      <span
                        key={i}
                        onClick={sectionSelect.bind(
                          this,
                          `${nowY}-${nowM}-${day}`
                        )}
                        className={styles.qujian}
                      >
                        {item}
                      </span>
                    );
                  } else {
                    return (
                      <span
                        key={i}
                        onClick={sectionSelect.bind(
                          this,
                          `${nowY}-${nowM}-${day}`
                        )}
                        className={`${styles.able} car-bg-hover__vi`}
                      >
                        {item}
                      </span>
                    );
                  }
                }
              } else {
                if (defaultValue.includes(`${nowY}-${nowM}-${day}`)) {
                  return (
                    <span
                      key={i}
                      onClick={sectionSelect.bind(
                        this,
                        `${nowY}-${nowM}-${day}`
                      )}
                      className={`${styles.selected} car-bg__vi`}
                    >
                      {item}
                    </span>
                  );
                } else {
                  return (
                    <span
                      key={i}
                      onClick={sectionSelect.bind(
                        this,
                        `${nowY}-${nowM}-${day}`
                      )}
                      className={`${styles.able} car-bg-hover__vi`}
                    >
                      {item}
                    </span>
                  );
                }
              }
            } else {
              return (
                <span key={i} className={styles.hui}>
                  {item}
                </span>
              );
            }
          })}
          {after.map((item, i) => {
            return (
              <span key={i} className={styles.hui}>
                {item}
              </span>
            );
          })}
        </div>
      </div>
    );
  }
}

export default Month;
