import html2canvas from 'html2canvas';
import canvg from 'canvg';

export function downBase(canvas, filename) {
  const base64 = canvas.toDataURL('image/png');
  const fileName = filename + '.png';

  const copyCanvas = document.createElement('canvas');
  const ctx = copyCanvas.getContext('2d');
  copyCanvas.width = canvas.width;
  copyCanvas.height = canvas.height;
  copyCanvas.setAttribute('style', canvas.getAttribute('style'));
  const img = new Image();
  img.src = base64;
  setTimeout(() => {
    ctx.drawImage(img, 0, 0);
    ctx.globalAlpha = window.userType === 7 ? 0.15 : 0.02;
    ctx.font = '400 14px Arial';
    ctx.textAlign = 'center';

    for (let i = 0; i <= canvas.width / 200 + 2; i++) {
      for (let j = 0; j <= canvas.height / 200 + 4; j++) {
        ctx.save();
        ctx.beginPath();
        ctx.translate(200 * i, 100 * j);
        ctx.rotate(-45 * Math.PI / 180);
        ctx.translate(-200 * i, -100 * j);
        ctx.fillText('汽车之家车智云', 200 * i, 100 * j);
        ctx.fillText(window.userName, 200 * i, 100 * j + 20);
        ctx.restore();
      }
    }

    if (navigator.msSaveOrOpenBlob) {
      let blob = copyCanvas.msToBlob();
      navigator.msSaveBlob(blob, fileName);
    } else {
      let eleA = document.createElement('a');
      eleA.href = copyCanvas.toDataURL('image/png');
      eleA.download = fileName;
      eleA.click();
      eleA = null;
    }
  }, 30);
}

export function dom2PNG(dom, filename, scale) {

  // let itm = document.getElementById('myList2').lastChild;
  let cln = dom.cloneNode(true);
  cln.style.width = dom.scrollWidth + 'px';
  cln.style.height = dom.scrollHeight + 'px';
  const downloadDom = document.getElementById('downloadimg');
  downloadDom.innerHTML = '';
  downloadDom.appendChild(cln);
  const options = {
    useCORS: true, logging: true, scale
  };
  html2canvas(cln, options)
    .then((canvas) => {
      downBase(canvas, filename);
    });
}

const svg2canvas = (dom) => {
  let nodesToRecover = [];
  let nodesToRemove = [];
  let svgElem = dom.querySelectorAll('svg');// divReport为需要截取成图片的dom的id
  for (const node of svgElem) {
    let parentNode = node.parentNode;
    let svg = (node.outerHTML || new XMLSerializer().serializeToString(node)).trim();
    let canvas = document.createElement('canvas');
    canvg(canvas, svg);
    if (node.style.position) {
      canvas.style.position += node.style.position;
      canvas.style.left += node.style.left;
      canvas.style.top += node.style.top;
    }
    nodesToRecover.push({
      parent: parentNode,
      child: node
    });
    parentNode.removeChild(node);
    nodesToRemove.push({
      parent: parentNode,
      child: canvas
    });
    parentNode.appendChild(canvas);
  }
  return dom;
};


export const dom2canvas = (dom, options) => {
  return new Promise((resolve, reject) => {
    let cln = dom.cloneNode(true);
    // 过滤canvas
    const preCanvas = dom.querySelectorAll('canvas');
    const canvasList = cln.querySelectorAll('canvas');
    [...canvasList].forEach((item, i) => {
      if (item.parentNode) {
        const img = new Image();
        img.src = preCanvas[i].toDataURL();
        img.width = preCanvas[i].width;
        img.height = preCanvas[i].height;
        img.style.maxWidth = '100%';
        img.style.maxHeight = '100%';
        item.parentNode.appendChild(img);
        item.parentNode.removeChild(item);
      }
    });
    // 过滤canvas end
    cln = svg2canvas(cln);
    cln.style.width = dom.scrollWidth + 'px';
    cln.style.height = dom.scrollHeight + 'px';
    const downloadDom = document.getElementById('downloadimg');
    if (downloadDom) {
      downloadDom.innerHTML = '';
      downloadDom.appendChild(cln);
      const newOptions = {
        useCORS: true,
        logging: true,
        scale: 1,
        ...options
      };
      html2canvas(cln, newOptions)
        .then((canvas) => {
          resolve(canvas);
        });
    } else {
      reject(new Error('dom null'));
    }
  });
};


function downLoadPNG(chartDom, fileName) {
  // const dd = svg2canvas(chartDom);
  setTimeout(() => {
    dom2canvas(chartDom).then(canvas => {
      downBase(canvas, fileName);
    });
    // if (navigator.msSaveOrOpenBlob) {
    //   dom2canvas(chartDom).then(canvas => {
    //     downBase(canvas, fileName);
    //   });
    // } else {
    //   console.log('----http://localhost:8001/----',chartDom.style.height)
    //   html2canvas(chartDom,
    //     {
    //       useCORS: true,
    //       logging: true,
    //       width: chartDom.style.width,
    //       height: chartDom.style.height,
    //       windowHeight: window.screen.availHeight,
    //       y: window.pageYOffset,
    //       scrollY: Element.scrollY,
    //       scale: 1,
    //     }).then((canvas) => {
    //       if (navigator.msSaveOrOpenBlob) {
    //         //const can = svg2canvas(canvas)
    //         let blob = canvas.msToBlob();
    //         navigator.msSaveBlob(blob, fileName);
    //       } else {
    //         let _img = canvas.toDataURL('image/png');
    //         let eleA = document.createElement('a');
    //         eleA.href = _img;
    //         eleA.download = fileName;
    //         eleA.click();
    //         eleA = null;
    //       }

    //     });
    // }

  }, 30)

  // dom2canvas(chartDom).then(canvas => {
  //   downBase(canvas, fileName);
  // });
}


export default downLoadPNG;
