import { Component } from 'react';
import html2canvas from 'html2canvas';
import canvg from 'canvg';
import Dorpdown from '_c/Button/Dropdown';
import DorpItem from '_c/Button/DropItem';

import downloadPng, { downBase } from './_downloadPng';
const typeObj = {
  1: { title: '下载PNG图片格式', titlezh: '下载PNG图片格式', titleen: "Download PNG", titlede: 'PNG Format', markType: 'png-download' }, // highChart 下载png
  2: { title: '下载PNG图片格式', titlezh: '下载PNG图片格式', titleen: "Download PNG", titlede: 'PNG Format', markType: 'png-download' }, // dom 下载png
  3: { title: '下载JPG图片格式', titlezh: '下载JPG图片格式', titleen: "Download", titlede: 'Download', markType: 'JPG' },
  4: { title: '下载JPG图片格式', titlezh: '下载JPG图片格式', titleen: "Download", titlede: 'Download', markType: 'JPG' },
  5: { titlezh: '下载Excel表格格式', titleen: "Download Excel", titlede: 'Excel Tabelle', markType: 'excel-download' }, //下载excel
  6: { titlezh: '添加至购物车', titleen: "Download", titlede: 'Download', markType: 'cart-download' },//highChart 添加到购物车
  7: { titlezh: '添加至购物车', titleen: "Download", titlede: 'Download', markType: 'cart-download' },//dom 添加到购物车
}
const langObj = {
  zh: '下载',
  en: 'Download',
  de: 'Download'
};
const svg2canvas = (dom) => {
  let nodesToRecover = [];
  let nodesToRemove = [];
  let svgElem = dom.querySelectorAll('svg');// divReport为需要截取成图片的dom的id
  for (const node of svgElem) {
    let parentNode = node.parentNode;
    let svg = (node.outerHTML || new XMLSerializer().serializeToString(node)).trim();
    let canvas = document.createElement('canvas');
    canvg(canvas, svg);
    if (node.style.position) {
      canvas.style.position += node.style.position;
      canvas.style.left += node.style.left;
      canvas.style.top += node.style.top;
    }
    nodesToRecover.push({
      parent: parentNode,
      child: node
    });
    parentNode.removeChild(node);
    nodesToRemove.push({
      parent: parentNode,
      child: canvas
    });
    parentNode.appendChild(canvas);
  }
  return dom;
};

class DownLoad extends Component {

  constructor(props) {
    super(props)
    this.state = {}
    this.domCanvas = null;
  }

  componentDidMount() {
    let domCanvas = document.querySelector('#temp__canvas__dom');
    if (!domCanvas) {
      domCanvas = document.createElement('canvas');
      domCanvas.id = 'temp__canvas__dom';
      domCanvas.style.display = 'none';
      document.body.appendChild(domCanvas);
    }
    this.domCanvas = domCanvas;
  }

  down = (value) => {
    const { chart, imgName, imgId, downUrl, chartDom } = this.props;
    switch (value) {
      case 1:
        //highChart 下载png
        this.export(chart, imgName);
        break;
      case 2:
        downloadPng(chartDom.current, imgName);
        //dom 下载png
        break;
      case 3:
        //highChart 下载jpg
        this.export(chart, imgName);
        break;
      case 4:
        //dom 下载jpg
        downloadPng(chart.current, imgName);
        break;
      case 5:
        //下载excel
        this.getExcel(downUrl);
        break
      case 6:
        //highChart 添加到购物车
        this.addToCart(imgId, chart, 1);
        break;
      case 7:
        //dom 添加到购物车
        this.addToCart(imgId, chart, 2);
        break;
      default:
    }
  }

  export = (c, n) => {
    if (c.exportChart) {
      setTimeout(() => {
        canvg(this.domCanvas, c.getSVG({
          exporting: {
            sourceWidth: c.chartWidth,
            sourceHeight: c.chartHeight
          }
        }));
        downBase(this.domCanvas, n);
      }, 1000);
    } else {
      downloadPng(c, n);
    }
  }

  getExcel = (downUrl) => {
    if (!downUrl) return false;
    location.href = downUrl;
  }



  addToCart = (id, chart, type) => {
    //type=1 highChart图 type=2 dom图
    let cimg = '';
    let bimg = '';
    const { addToCart } = this.props;
    if (type === 2) {
      cimg = chart.current;
      const dd = svg2canvas(cimg);
      html2canvas(dd).then(canvas => {
        bimg = canvas.toDataURL("image/png");
        addToCart(bimg);
      });
    } else {
      cimg = canvg(document.getElementById('cartImgArea'), chart.getSVG(
        {
          exporting: {
            sourceWidth: chart.chartWidth,
            sourceHeight: chart.chartHeight
          }
        }
      ));
      bimg = document.getElementById('cartImgArea').toDataURL("image/png");
      addToCart(bimg);
    }
  }

  render() {
    const { types, style, imgName, pmark, lang } = this.props;
    // 'PNG', 'EXCEL',  'ADDCART',
    return (
      <React.Fragment>
        <Dorpdown
          data-role="download"
          style={{ ...style, float: 'right' }}
          size={'small'}
          icon={'xiazai'}
          title={langObj[lang]}
          placement='right'>
          {types.map((item, i) => {
            return (
              <DorpItem
                data-click={{ title: imgName, pmark, type: typeObj[item].markType }}
                key={`down${Math.random() * 10}`}
                onClick={this.down.bind(this, item)}>
                {typeObj[item][`title${lang}`]}
              </DorpItem>)
          })}
        </Dorpdown>
      </React.Fragment>
    )
  }
}
DownLoad.defaultProps = {
  lable: {},
  extras: '',
  param: {},
  lang: 'zh'
}

export default DownLoad;
