import React, { Component } from 'react';
import PropTypes from 'prop-types';
import styles from './index.scss';
import Icon from '../Icon';
import Button from '../Button/Button.js';
import _event from '../event';
import { Checkbox } from 'antd';
const getNode = (node) => {
  if (node.tagName === 'BODY') {
    return true;
  } else if (node.hasAttribute('data-drop')) {
    return false;
  } else {
    return getNode(node.parentNode)
  }
}

class Dorpdown extends Component {
  constructor(props, content) {
    super(props);
    this.handleOpen = this.handleOpen.bind(this);
    this.state = {
      open: false,
      active: false,
      value: []
    }
  }

  // 销毁事件
  componentWillUnmount() {
    _event.removeEvent(document, 'click', this.dropBlur);
  }

  // 绑定d点击组件外 隐藏当前下拉事件
  componentDidMount() {
    this.setState({ 'activeItem': this.props.value });
    _event.addEvent(document, 'click', this.dropBlur);
  }

  handleOpen(e) {
    this.setState((prevState) => {
      return {
        active: true
      }
    })

    if (this.state.open) {
      this.setState((prevState) => {
        return {
          open: !this.state.open,
          active: false
        }
      })
    } else {
      this.setState({ open: true });
    }
  }

  dropBlur = (e) => {
    if (!getNode(e.target)) return;
    else {
      this.setState({ open: false });
    }
  }

  onCheckedChange = (checkedValues) => {
    this.setState({ value: checkedValues });
    const { onChange } = this.props;

    if (onChange) {
      onChange(checkedValues);
    }
  }

  render() {
    const { size, title, children, style, placement, defaultValue, ...other } = this.props;
    const { open } = this.state;
    const placeStyle = placement == 'right'
      ? {
        'right': 0,
      }
      : {
        'left': 0
      };
    const itemList = React.Children.map(children, (item) => {
      if (item) {
        return React.cloneElement(item);
      } else {
        return '';
      }
    });
    //下拉框打开样式
    const openCls = this.state.open && this.props.icon === 'xiala'
      ? 'car-select-dropdown-open'
      : '';
    const openStyle = open ? { 'display': 'block' } : { 'display': 'none' };
    return (
      <div
        {...other}
        style={style}
        className={`${styles['car-dorpdown']} ${styles[openCls]}`}
        tabIndex={-1}
        data-drop={'drop-mul'}>
        <Button
          type={'default'}
          onClick={this.handleOpen}
          size={'small'}>
          {title}
          <Icon type='xiafanye' />
        </Button>
        {
          <Checkbox.Group
            defaultValue={defaultValue}
            style={{ width: '100%', ...placeStyle, ...openStyle }}
            className={styles['car-checkbox-con']}
            onChange={this.onCheckedChange}>
            <ul
              className={styles['car-dorp-items']}>
              {itemList}
            </ul>
          </Checkbox.Group>
        }
      </div>)
  }
}
export default Dorpdown;
