import React, { Component, Fragment } from 'react';
import styles from './style.scss';
import PropTypes from 'prop-types';
import { ChartItem, Loading } from '../Chart';
import Icon from '../Icon';
import chartOption from './option';

class Fiveline extends Component {
  state = {
    colors: [{
      background: 'rgba(251, 149, 0, 0.08)',
      color: 'rgb(251, 149, 0)',
      borderColor: 'rgba(251, 149, 0, 0.2)'
    }, {
      background: 'rgba(52, 54, 199, 0.08)',
      color: 'rgb(52, 54, 199)',
      borderColor: 'rgba(52, 54, 199, 0.2)'
    }, {
      background: 'rgba(54, 213, 168, 0.08)',
      color: 'rgb(54, 213, 168)',
      borderColor: 'rgba(54, 213, 168, 0.2)'
    }, {
      background: 'rgba(255, 95, 95, 0.08)',
      color: 'rgb(255, 95, 95)',
      borderColor: 'rgba(255, 95, 95, 0.2)'
    }, {
      background: 'rgba(162, 28, 255, 0.08)',
      color: 'rgb(162, 28, 255)',
      borderColor: 'rgba(162, 28, 255, 0.2)'
    },],
    chartColors: ['#FB9500', '#3436C7', '#36D5A8', '#FF5F5F', '#A21CFF'],
    modalShow: false,
    datas: [],
    chartLoading: false,
    detailList: []
  }

  componentDidMount() {
    const { charts, fiveType } = this.props;
    const { chartColors } = this.state;
    const { body, header } = charts;
    let _this = this;

    let datas = body.map((item, i) => {
      let tempDatas = item;

      let temp = {
        ...JSON.parse(JSON.stringify(chartOption)),
        colors: [chartColors[i]],
        series: [{ data: item }]
      }

      if (fiveType == 1) {
        temp.plotOptions.spline.dataLabels = {
          allowOverlap: false,
          enabled: true,
          format: '{y:.2f}%',
          // 标签样式
          style: {
            textOutline: 'none',
            color: '#333'
          }
        }
      }

      temp.xAxis.categories = header;
      if (i < 2) {
        temp.plotOptions.spline.point = {
          events: {
            click: function (event) {
              let params = {
                selector: i + 1,
                startDate: event.point.category
              }
              _this.floatShow(params)
            }
          }
        }
      }
      return temp
    })

    this.setState({ datas })
  }
  // 缓存图表
  setChartList = () => {

  }

  saveChart = (obj) => {
    const { saveCharts } = this.props;
    saveCharts(obj)
  }

  // 曲线图上的点击
  floatShow = async (params) => {
    await this.setState({ modalShow: true, chartLoading: true });

    const { pointServer } = this.props;
    let data = await pointServer(params)

    this.setState({ chartLoading: false, detailList: data })
  }

  closeModal = () => {
    this.setState({ modalShow: false })
  }

  render() {
    const { lables, charts } = this.props;
    const { colors, modalShow, datas, chartLoading, detailList } = this.state;
    const { header } = charts;

    const _this = this;

    const showList = () => {
      return detailList.map((item, i) => {
        return (
          <a className={styles['list_1']} href={item.url} key={i} target='_blank'>
            <p>{item.title}</p>
            <p>{item.data}</p>
          </a>
        )
      })
    }
    return (
      <div className={styles['Fiveline']}>
        {
          modalShow && (
            <div className={styles['modal']}>
              <div className={styles['close']} onClick={this.closeModal}>
                <Icon type='guanbi' />
              </div>
              {
                chartLoading ? (
                  <div className={styles['loading']}>
                    <Loading />
                  </div>
                ) : (
                    <Fragment>
                      <div className={styles['list']}>
                        <p>内容标题</p>
                        <p>关注人数</p>
                      </div>
                      {
                        detailList.length == 0 ? (
                          <div className={styles['nodata']}>
                            <p>暂无数据</p>
                          </div>
                        ) : showList()
                      }
                    </Fragment>
                  )
              }
            </div>
          )
        }


        {/* 左侧标签 */}
        <div className={styles['left']}>
          {
            lables.map((item, i) => {
              return (
                <div className={styles['left-item']} key={i}>
                  <p style={colors[i]}>{item}</p>
                </div>
              )
            })
          }
        </div>

        {/*  右侧五线图 */}
        <div className={styles['right']}>
          {
            datas.map((item, i) => {
              if (i > 4) return;

              let cls = styles['right-chart'];
              let _h = 100;

              if (i == 4) {
                _h = 120
                item.xAxis.visible = true;
                cls = styles['right-chart1']
              }

              if (i == 4) {
                return (
                  <div className={cls} key={i}>
                    <ChartItem
                      style={{ height: _h, width: '100%' }}
                      option={item}
                      loadOk={this.saveChart}
                    />
                  </div>
                )
              } else {
                return (
                  <div className={cls} key={i}>
                    <ChartItem
                      style={{ height: _h, width: '100%' }}
                      option={item}
                      loadOk={this.setChartList}
                    />
                  </div>
                )
              }
            })
          }
        </div>

      </div>
    )
  }
}
export default Fiveline;
