import React, { Component } from 'react'
import PropTypes from 'prop-types'
import styles from './index.scss'
import Icon from './../Icon'
import SubWrap from './ColSubWrap'
import ColSubMenu from './ColSubMenu'
class SubMenu extends Component {
  constructor(props, content) {
    super(props);
    this.toggleActive = this.toggleActive.bind(this);
    this.handleActiveSub = this.handleActiveSub.bind(this);
    this.titleREF = React.createRef();
    this.state = {
      open: false,
      activeSubItem: '',
      subOverOn: false,
      hh: 60,
      onLevel: 0,
    }
  }
  async toggleActive(e) {
    const { collapsed, activeSub, val, resetActiveSub } = this.props;
    if (collapsed) return false;
    if (activeSub.indexOf(val) !== -1) {
      let na = [];
      await activeSub.forEach((val, index) => {
        if (val !== this.props.val) {
          na.push(val);
        }
      });
      resetActiveSub(na);
    } else {
      resetActiveSub([...activeSub, val]);
    }

  }
  subOver = (e, level) => {
    e.stopPropagation();
    const { collapsed, val } = this.props;
    if (!collapsed) return false;
    const children = this.props.children;
    const hall = this.titleREF.current.offsetTop - document.getElementById('car-menu-con').scrollTop + 36 * children.length;
    const hcon = document.getElementById('car-menu-con').offsetHeight;
    let hh = 0;
    if (hall - hcon > 0) {
      hh = hcon - 36 * children.length;
    } else {
      hh = this.titleREF.current.offsetTop - document.getElementById('car-menu-con').scrollTop;
    }
    this.setState({ subOverOn: true, hh, onLevel: level });
  }
  subOut = (e, level) => {
    e.stopPropagation();
    const { collapsed } = this.props;
    if (!collapsed || level != this.state.onLevel) return false;
    setTimeout(() => {
      if (level == this.state.onLevel) {
        this.setState({ subOverOn: false })
      }
    }, 0);
  }
  componentWillMount() {
    this.setState({ 'activeSub': this.props.defaultActiveSub, 'activeKey': this.props.defaultActiveItem });
  }

  handleActiveSub = (val) => {
    this.setState({ 'activeSubItem': val })
  }
  render() {
    const children = this.props.children;

    const hiddenCls = this.props.activeSub.indexOf(this.props.val) !== -1
      ? 'car-menu-active'
      : 'car-menu-hidden'
    const arrowCls = this.props.activeSub.indexOf(this.props.val) == -1
      ? 'car-arrow-slid'
      : 'car-arrow-open'
    const padNum2 = 20;
    const padNum3 = 46;
    const padNum = this.props.level == 2
      ? padNum2
      : padNum3;
    const padStyle = {
      paddingLeft: padNum + 'px'
    };
    const { collapsed, activeSub, val } = this.props;
    const items = React.Children.map(children, (item) => {
      if (item.props.children instanceof Array) {
        return React.cloneElement(item, {
          activeSub: this.state.activeSubItem,
          level: this.props.level + 1,
          resetActiveSub: this.handleActiveSub,
          activeItem: this.props.activeKey,
          resetActiveKey: this.props.resetActiveKey,
          activeKey: this.props.activeKey,
          defaultActiveSub: this.props.defaultActiveSub,
          collapsed: this.props.collapsed
        })
      } else {
        return React.cloneElement(item, {
          activeItem: this.props.activeKey,
          level: this.props.level + 1,
          activeKey: this.props.activeKey,
          resetActiveKey: this.props.resetActiveKey,
          collapsed: this.props.collapsed
        })
      }

    })
    const ckeys = items.map((item) => {
      return item.props.val;
    })

    const colIconClm = ((collapsed && activeSub == val) || this.state.subOverOn || ckeys.indexOf(this.props.activeKey) >= 0) ? 'col_icon_clm' : '';
    return (<li className={styles['car-menu-sub']}>
      <div
        ref={this.titleREF}
        style={padStyle}
        className={`${styles['car-menu-sub-title']}  ${styles[colIconClm]}`}
        onClick={(e) => this.toggleActive(e)}
        onMouseOver={(e, l) => this.subOver(e, 1, this.titleREF)}
        onMouseLeave={(e, l) => this.subOut(e, 1)}>

        {this.props.icon &&
          <Icon
            type={this.props.icon}
            clm={`${styles['menu-title-icon']}  ${styles[colIconClm]}`}>
          </Icon>
        }
        <span className={styles['menu-title']}>{this.props.title}</span>

        {items &&
          <Icon
            type={'xiala'}
            clm={`${styles['menu-arrow-icon']}
              ${styles[arrowCls]}`}>
          </Icon>}

        {collapsed && this.state.subOverOn &&
          <SubWrap>
            <ColSubMenu
              style={{ top: this.state.hh + 60 }}
              top={this.state.hh}
              level={2}
              title={this.props.title}
              items={items}
              onSubOver={this.subOver}
              onSubLeave={this.subOut}
              activeKey={this.props.activeKey}
              subVal={this.props.val}
              resetActiveKey={this.props.resetActiveKey}
              resetActiveSub={this.props.resetActiveSub}>
            </ColSubMenu>
          </SubWrap>
        }
      </div>
      {
        !collapsed &&
        <ul className={`${styles['car-menu-sub-con']} ${styles[hiddenCls]}`}>
          {items}
        </ul>
      }

    </li>)
  }
}
export default SubMenu;
