import React, { Component } from 'react';
import classnames from 'classnames';
import styles from './index.less';


const priceList: any[] = [];

for (let i = 0; i < 7; i++) {
  let item: number[] = [];
  for (let j = 1; j <= 5; j++) {
    item.push(i * 5 + j);
  }
  priceList.push(item);
}

priceList.push([38, 40, 45, 48, 50]);
priceList.push([55, 60, 65, 70]);
priceList.push([80, 90, 100]);

interface ICompeteProps {
  startPrice: number;
  endPrice: number;
  priceChange: any;
  minPrice?: number;
  maxPrice?: number;
}

interface ICompeteState {
  rangeWidth: number;
  startPoint: number;
  endPoint: number;
  startPrice: number;
  endPrice: number;
  isDown: boolean;
  offsetLeftList: any[];
  barLeft: number;
}

class PriceSelect extends Component<ICompeteProps, ICompeteState> {
  // public splitList: React.RefObject<{}>;
  public splitList: React.RefObject<HTMLInputElement>;
  public constructor(props: ICompeteProps) {
    super(props);
    this.splitList = React.createRef();
    this.state = {
      rangeWidth: 360,
      startPoint: 0,
      endPoint: 0,
      startPrice: props.startPrice || 2,
      endPrice: props.endPrice || 26,
      isDown: false,
      offsetLeftList: [],
      barLeft: 0,
    };

  }

  public send() {
    const { startPrice, endPrice } = this.state;

    this.props.priceChange &&
      this.props.priceChange({
        startPrice: startPrice > endPrice ? endPrice : startPrice,
        endPrice: startPrice < endPrice ? endPrice : startPrice,
      });
  }

  public async splitClick(e: any, val: any) {
    const { endPoint, startPoint, barLeft } = this.state;
    const distance: number = e.clientX - barLeft;
    let point = 'endPoint';
    let price = 'endPrice';
    // 点击的地方距离开始比较近
    if (Math.abs(distance - endPoint) > Math.abs(distance - startPoint)) {
      price = 'startPrice';
      point = 'startPoint';
    }

    let state = {};
    state[price] = val;
    state[point] = distance - 4;
    await this.setState(state);

    this.send();
  }

  public slider(e: any, type: string, direction: string) {
    const { minPrice = 0, maxPrice = 100 } = this.props;
    const { offsetLeftList, rangeWidth, barLeft } = this.state;
    const minLeft = (offsetLeftList.find(item => item.price === minPrice) || { left: 0 }).left + 0.5;
    const maxLeft = (offsetLeftList.find(item => item.price === maxPrice) || { left: offsetLeftList[offsetLeftList.length - 1].left }).left + 1;
    let move: string = type === 'PC' ? 'mousemove' : 'touchmove';
    let end: string = type === 'PC' ? 'mouseup' : 'touchend';
    this.setState({ isDown: true });

    let moveFunc: any = (moveE: any): void => {
      let currentX: number = moveE.clientX - barLeft - 4;
      if (currentX < (minLeft || 0)) currentX = minLeft || 0;
      if (currentX > (maxLeft || rangeWidth)) currentX = (maxLeft || rangeWidth);

      const isLeft: boolean = direction === 'left';
      const key: string = isLeft ? 'startPrice' : 'endPrice';
      let val = 0;
      for (let i = 0; i < offsetLeftList.length; i++) {
        let rangeLeft: boolean = currentX >= offsetLeftList[i].left;
        let rangeRight: boolean = currentX <= (offsetLeftList[i + 1] || { left: 10000 }).left;
        if (rangeLeft && rangeRight) {
          val = offsetLeftList[i].price;
          break;
        }
      }

      if ([0, 0.5].includes(currentX)) val = 0;
      const nn: string = isLeft ? 'startPoint' : 'endPoint';
      let state = {};
      state[nn] = currentX;
      state[key] = val;
      this.setState(state);
    };
    let endFunc: any = (endE: any): void => {
      document.removeEventListener(move, moveFunc);
      document.removeEventListener(end, endFunc);
      moveFunc = endFunc = null;
      this.setState({ isDown: false }, this.send);
    };
    document.addEventListener(move, moveFunc);
    document.addEventListener(end, endFunc);
  }

  public initPoint() {
    const { startPrice, endPrice, offsetLeftList } = this.state;
    offsetLeftList.forEach((item: any) => {
      if (startPrice === item.price) this.setState({ startPoint: item.left });
      if (endPrice === item.price) this.setState({ endPoint: item.left });
    });
  }

  public initBar() {
    const splitList: any = this.splitList.current;
    const bound = splitList.getBoundingClientRect();
    this.setState({ barLeft: (bound.x || bound.left) });
    this.setState({ offsetLeftList: [] });

    const priceArr: any[] = priceList.join(',').split(',');
    [...splitList.querySelectorAll('div span')].forEach((item, i) => {
      let ofl = this.state.offsetLeftList;
      ofl.push({
        left: item.offsetLeft,
        price: Number(priceArr[i])
      });
      this.setState({ offsetLeftList: ofl });
    });

    this.initPoint();
  }

  public componentDidMount() {
    // this.send();
    setTimeout(this.initBar.bind(this), 0);
  }
  public static async getDerivedStateFromProps(nextProps: ICompeteProps, prevState: ICompeteState) {
    const { startPrice, endPrice } = nextProps;
    if (
      nextProps.startPrice !== prevState.startPrice
      ||
      nextProps.endPrice !== prevState.endPrice
    ) {
      const { offsetLeftList } = prevState;
      let startPoint = 0;
      let endPoint = 0;
      offsetLeftList.forEach((item: any) => {
        if (startPrice === item.price) startPoint = item.left;
        if (endPrice === item.price) endPoint = item.left;
      });
      return { startPrice: nextProps.startPrice, endPrice: nextProps.endPrice, startPoint: startPoint, endPoint: endPoint };
    } else {
      return null;
    }
  }

  public render() {

    const {
      rangeWidth, startPoint, endPoint,
      isDown, startPrice, endPrice,
    } = this.state;

    const { minPrice = 0, maxPrice = 100 } = this.props;
    return (
      <section style={{ height: '65px' }}>
        <div className={styles.priceselect} style={{ width: 360 }}>
          <section ref={this.splitList}>
            {
              priceList.map(item => (
                <div key={item.join('-')} className={styles.piece}>
                  {
                    item.map((val: any) => (
                      <span
                        key={val + '_price'}
                        className={classnames(styles.split, {
                          [styles.disabled]: minPrice > val || maxPrice < val
                        })}
                        style={{ width: 10 / item.length * 10 + '%' }}
                        title={`${val}万`}
                        onClick={(e) => { this.splitClick(e, val) }}
                      />
                    ))
                  }
                </div>
              ))
            }
          </section>

          <span className={styles.poles} style={{ left: 0 }}>0万</span>
          <span className={styles.poles} style={{ right: -10 }}>100万</span>

          <div
            className={styles.start}
            style={{ left: startPoint }}
            onMouseDown={(e) => { this.slider(e, 'PC', 'left') }}
            onTouchStart={(e) => { this.slider(e.touches[0], 'M', 'left') }}
          >
            <span className={styles.tip} style={{ opacity: isDown ? 1 : 0 }}>{startPrice}万</span>
          </div>
          <div
            className={styles.end}
            style={{ left: endPoint }}
            onMouseDown={(e) => { this.slider(e, 'PC', 'right') }}
            onTouchStart={(e) => { this.slider(e.touches[0], 'M', 'right') }}
          >
            <span className={styles.tip} style={{ opacity: isDown ? 1 : 0 }}>{endPrice}万</span>
          </div>

          <div className={styles.bar} style={{ left: (startPoint > endPoint ? endPoint : startPoint) + 2, width: Math.abs(endPoint - startPoint) }}>
            <span
              className={styles.tip}
              style={{ top: -12, opacity: isDown ? 0 : 1 }}
            >
              {[startPrice, endPrice].sort((n, m) => n - m).join('-')}万
            </span>
          </div>
        </div>
      </section>
    );
  }
}

export default PriceSelect;
