import React, { Component } from 'react';
import PropTypes from 'prop-types';
import styles from './index.scss';
import Icon from './../Icon'
import _event from '../event';

class Select extends Component {
  constructor(props, content) {
    super(props);
    let id = Math.floor(Math.random() * 10000) + Date.now();
    this.handleActiveItem = this.handleActiveItem.bind(this);
    this.handleOpen = this.handleOpen.bind(this);
    this.dropBlur = this.dropBlur.bind(this);
    this.state = {
      activeItem: '', //选中的
      open: false, //下拉是否打开
      text: '', //显示的文字
      value: '', //选中的值
      id,
      active: false
    }
  }
  handleActiveItem(val, text, object) {
    this.setState({ 'activeItem': val, 'text': text, 'value': val, 'open': false });
    const value = object
      ? object
      : val;
    if (this.props.onChange) {
      this.props.onChange(value, text);
    }
  }
  dropBlur() {
    if (this.state.active) {
      this.setState({ 'open': true, 'active': false });
    } else {
      this.setState({ 'open': false, 'active': false });
    }
  }
  handleOpen(e) {
    this.setState((prevState) => {
      return {
        open: !this.state.open,
        active: true
      }
    })

    if (this.state.open) {
      this.setState((prevState) => {
        return {
          open: !this.state.open,
          active: false
        }
      })
    }

    if (this.props.visibleChange) {
      this.props.visibleChange(!this.state.open);
    }

  }
  // 销毁事件
  componentWillUnmount() {
    _event.removeEvent(document, 'click', this.dropBlur);
  }
  componentWillMount() {
    if (this.props.fix) {
      this.setState({ 'activeItem': this.props.value, text: this.props.value, 'value': this.props.value });
    } else {
      this.setState({ 'activeItem': this.props.value, text: this.props.value, 'value': this.props.value });
    }

    _event.addEvent(document, 'click', this.dropBlur);
  }

  //每当我们通过父组件更新子组件props时（这个也是唯一途径），这个方法就会被调用。
  componentWillReceiveProps(nextProps) {
    if (nextProps.value != this.props.value) {
      this.setState({ 'activeItem': nextProps.value, 'text': nextProps.value, value: nextProps.value });
      if (this.props.onChange && this.props.dfchange) {
        this.props.onChange(nextProps.value);
      }
    }
  }

  render() {
    const children = this.props.children;
    const options = React.Children.map(children, (child) => {
      if (child == null) {
        return ''
      } else {
        return React.cloneElement(child, {
          activeItem: this.state.activeItem,
          setActiveItem: this.handleActiveItem,
          size: this.props.size ? this.props.size : 'default',
          fix: this.props.fix
        })
      }

    })

    const { id } = this.state;
    //下拉框打开样式
    const openCls = this.state.open
      ? 'car-select-dropdown-open'
      : '';

    const _cls = this.state.open
      ? 'car-border__vi'
      : '';

    //选择的内容
    const selectionTitle = this.state.text != ''
      ? this.state.text
      : this.props.placeholder;

    const sizeStyle = this.props.size === 'small' ? { height: 24, lineHeight: '24px' } : {};

    const { width, style, onChange, size, fix, dfchange, value, visibleChange, ...other } = this.props;

    return (
      <div
        {...other}
        tabIndex="1"
        data-drop='select'
        className={`${styles['car-select-con']}  ${styles[openCls]}`}
        style={{ width: this.props.width, ...this.props.style }}>
        <div
          id={id}
          onClick={this.handleOpen}
          className={`${styles['car-select-selection']} car-border-hover__vi ${_cls}`}
          style={sizeStyle}>
          <span style={sizeStyle}>{selectionTitle}</span>
          <Icon type={'xiala'} style={sizeStyle}></Icon>
        </div>
        <ul
          className={`${styles['car-select-dropdown']}`}
          data-drop='select'>
          {options}
        </ul>
      </div>)
  }
}
export default Select
