import { Component } from 'react';
import styles from './style.scss';
import _event from '../event';

// 点击外隐藏
const getNode = (node) => {
  if (node.tagName === 'BODY') {
    return true;
  } else if (node.hasAttribute('data-drop') && node.getAttribute('data-drop') === 'SeriesListWithSpec') {
    return false;
  } else {
    return getNode(node.parentNode);
  }
};

class SeriesListWithSpec extends Component {
  constructor(props) {
    super(props);
    this.state = {
      level1List: props.seriesLst,
      level2List: null,
      level3List: null,
      showSelect: false
    };
    this.stateObj = {
      '0': '未售',
      '10': '待售',
      '20': '在售',
      '30': '停产在售',
      '40': '停产'
    };
  }

  static getDerivedStateFromProps(props, state) {
    if (props.seriesLst !== state.level1List) {
      const newState = {
        level1List: props.seriesLst,
        level2List: null,
        level3List: null,
        showSelect: props.toHide
      };
      return newState;
    }
  }

  navTo = (key) => {
    const top = this[`ref_info-${key}`].offsetTop;
    this.ref_info.scrollTop = top - 30;
  };

  onSecondListClick = (val) => {
    const { dontShowLvl3 } = this.props;
    if (!val.children || dontShowLvl3) {
      this.props.onChange(val.id, val);
      this.setState({ showSelect: false })
    } else {
      this.setState({ level3List: val });
    }
  };

  onChange = (val) => {
    const { level3List } = this.state;
    this.props.onChange(val.id, val, level3List);
    this.setState({ showSelect: false })
  };

  hideDrop = (e) => {
    let now = e.target.parentNode;
    let temp = getNode(now);
    this.setState((state) => {
      if (!temp) { return null }
      _event.removeEvent(document.body, 'click', this.hideDrop);
      return { showSelect: false };
    });
  };

  showSelect = () => {
    if (this.state.showSelect) {
      _event.removeEvent(document.body, 'click', this.hideDrop);
    } else {
      this.setState({ showSelect: true })
      _event.addEvent(document.body, 'click', this.hideDrop);
    }
  }


  render() {
    const { level1List, level2List, level3List, showSelect } = this.state;
    const { title, style = {}, toHide } = this.props;
    return (
      <div className={styles.serieslist} tabIndex="-1"
        ref={(_tabi) => { this.reftabindex = _tabi }}>
        <div onClick={this.showSelect}
          className={styles.choose}
          style={style}
        >{title || '选择车型'}</div>
        <div
          data-drop="SeriesListWithSpec"
          className={styles.list}
          style={{ width: level3List ? 300 : 'auto', display: showSelect ? 'block' : 'none' }}>
          <p className={styles.title}>
            <span
              onClick={() => this.setState({ level2List: null, level3List: null })}>品牌</span>
            {level2List && ' > '}
            {level2List &&
              <span
                onClick={() => this.setState({ level3List: null })}
              >车系</span>}
            {level3List && ' > '}
            {level3List && <span>车型</span>}
          </p>
          {
            (!level2List && !level3List) &&
            <section key={1}>
              <ul className={styles.nav}>
                {
                  level1List.map((val, ind) => (<li key={ind}
                    onClick={() => { this.navTo(val.key) }}>
                    {val.label}
                  </li>))
                }
              </ul>
              <div className={styles.info}
                ref={(_info) => { this.ref_info = _info }}
                style={{ scrollBehavior: 'smooth' }}>
                {
                  level1List.map(item => (
                    <React.Fragment>
                      <h4 key={item.key} ref={(_info) => { this[`ref_info-${item.key}`] = _info }}>{item.label}</h4>
                      {
                        item.children.map(val => <p key={val.id}
                          onClick={() => { this.setState({ level2List: val }) }}>
                          {val.name}
                        </p>)
                      }
                    </React.Fragment>
                  ))
                }
              </div>
            </section>
          }

          {
            (!!level2List && !level3List) &&
            <section key={2}>
              <div className={styles.info}>
                {
                  level2List.children.map(item => (
                    <React.Fragment>
                      <h4 key={item.key} style={{ paddingLeft: 20 }}>{item.label}</h4>
                      {
                        item.children.map(val => <p
                          key={val.id}
                          style={{ paddingLeft: 20 }}
                          onClick={this.onSecondListClick.bind(null, val)}>
                          {val.name}
                        </p>)
                      }
                    </React.Fragment>
                  ))
                }
              </div>
            </section>
          }
          {
            (!!level3List && level3List.children) &&
            <section key={3}>
              <div className={styles.info} style={{ width: 300 }}>
                {
                  level3List.children.map(item => (
                    <React.Fragment>
                      <h4 key={item.key} style={{ paddingLeft: 20 }}>{this.stateObj[item.label]}</h4>
                      {
                        item.children.map(val => <p
                          key={val.id}
                          style={{ paddingLeft: 20 }}
                          onClick={this.onChange.bind(null, val)}>
                          {val.name}
                          <span>{`${(val.price / 10000).toFixed(2)}万`}</span>
                        </p>)
                      }
                    </React.Fragment>
                  ))
                }
              </div>
            </section>
          }
        </div>
      </div >
    );
  }
}

export default SeriesListWithSpec;
