import React, {Component} from 'react'
import styles from './index.scss'
import PropTypes from 'prop-types'
import _event from '../event';

class Tabs extends Component {

  //在这里进行初始化state和绑定this的工作
  constructor(props, content) {
    super(props)
    this.state = {
      activeKey: this.props.defaultValue || this.props.children[0]['props']['value'],
      scrollActive: false,
      fixedActive: false,
    }
  }
  handleClick = (key) => {
    this.setState({activeKey: key}, ()=>{
      if (this.props.onChange) {
        this.props.onChange(key);
      }
    });

  }

  setScrollTop = () => {
    let top = document.documentElement.scrollTop || document.body.scrollTop;
    if (top >= this._top) {
      this.setState({ scrollActive: true })
    } else {
      this.setState({ scrollActive: false })
    }
  }

  fixedScrollTop = () => {
    let top = document.documentElement.scrollTop || document.body.scrollTop;
    if (top >= this._top) {
      this.setState({ fixedActive: true })
    } else {
      this.setState({ fixedActive: false })
    }
  }

  componentDidMount() {
    // this.setState({
    //   'activeKey': this.props.children[0]['props']['value']
    // })
    const { getTopActive, fixedTopClassName, stickyTop } = this.props;

    if (getTopActive) {
      this._top = getTop(this._tabs) + this._tabs.clientHeight;
      document.addEventListener('scroll', this.setScrollTop);
    }

    function getTop(elem){
      let top = elem.offsetTop;
      let cur = elem.offsetParent;
      while (cur != null) {
        top += cur.offsetTop;
        cur = cur.offsetParent;
      }
      return top;
    }

    if (stickyTop && fixedTopClassName) {
      const stickyEle = document.querySelector('#__vi_tabs');
      if (getComputedStyle(stickyEle).position !== 'sticky') {
        this._top = getTop(stickyEle) - stickyTop;
        document.addEventListener('scroll', this.fixedScrollTop);
      }
    }
  }

  componentWillUnmount() {
    document.removeEventListener('scroll', this.setScrollTop);
    document.removeEventListener('scroll', this.fixedScrollTop);
  }

  render() {
    const { children, getTopActive, top, pTop, value, stickyTop, fixedTopClassName, titleStyle = {} } = this.props;

    const { scrollActive, fixedActive } = this.state;
    const childrenP = React.Children.toArray(this.props.children);
    const cf = childrenP.filter((item) => {
      return item !== false;
    });

    let tempActive = value ? value : this.state.activeKey;


    //标签列表
    const titles = React.Children.map(cf, (item, index) => {
      if(!item) return '';
      let titleObj = {
        'title': item.props.title,
        'key': item.props.value
      };
      let active = titleObj.key == tempActive
        ? 'active'
        : ''
      let _cls = titleObj.key == tempActive
        ? 'car-tabs-color__vi' : ''

      let temp = {};
      for (let x in item.props) {
        if (/^data\-\w+/.test(x)) {
          temp[x] = JSON.stringify(item.props[x])
        }
      }
      return (<li {...temp} key={`tabtitle${index}`} className={`${styles[active]} ${_cls}`} onClick={this.handleClick.bind(this, titleObj.key)}>{titleObj.title}</li>)
    })
    //面板列表
    const paneItem = React.Children.map(cf, (item) => {
      let activePane = '';
      if (item != null && item != undefined && item.props.value === tempActive) {
        activePane = item;
      }
      return activePane;
    })

    let smallCls = this.props.size == 'small'
      ? 'car-tabs-small'
      : ''

    let tabsClass = `${styles['car-tabs']} ${styles[smallCls]}`;
    let titlesClass = styles['car-tabs-nav-list'];

    let cls = scrollActive ? styles.tofix : '';

    return (
      <div className={tabsClass}>
        <section className={fixedActive ? fixedTopClassName : null } id="__vi_tabs" style={stickyTop ? { position: 'sticky', top: stickyTop, zIndex: 99 } : null}>
          <ul
            ref={tabs => this._tabs = tabs}
            className={`${titlesClass} ${cls}`}
            style={{ top: top, paddingTop: pTop, ...titleStyle }}>
            {titles}
          </ul>
        </section>
        {
          (scrollActive || fixedActive) &&
          <div className={styles.zhanwei}style={{ paddingTop: pTop }}></div>
        }
        <div ref={pane => this._pane = pane} className={styles['car-pane-con']}>
          {paneItem}
        </div>
      </div>
    )
  }
}

export default Tabs;
