import React, {Component, Fragment} from 'react';
import styles from './style.scss';
import PropTypes from 'prop-types';
import Tag from './Tag';

class Funnel extends Component {

  state ={
    globalOffsetX: null,
    drawOffsetX: null,
    colors: [
      '#6ECCFF',
      '#6ECCFF',
      '#6ECCFF',
      '#45B8FF',
      '#55ACEE',
      '#0D7DD9'
    ]
  }

  componentDidMount(){
    const {offsetX} = this.props;
    let angel = Math.atan2(35,offsetX);
    let jaodu = angel * 180 / Math.PI;
    let globalOffsetX = Math.abs(20 / Math.tan( jaodu ));
    let drawOffsetX = Math.abs(50 / Math.tan( jaodu ));

    this.setState({globalOffsetX, drawOffsetX})
  }

  render(){
    const {lables, tables, width} = this.props;
    const { globalOffsetX, drawOffsetX , colors} = this.state;

    let {heads, bodys} = tables;

    let _width;
    if(heads.length > 5){
      _width = heads.length * 150 + 'px';
    }else{
      _width = '100%'
    }

    let _height = lables.length * 75

    return (
      <div className={styles.Funnel} style={{ height: `${_height}px` }}>
        <svg
          className={styles.left}
          height={_height}
          width={width}
          style={{
            width: `${width}px`,
            height: `${_height}px`,
          }}
        >
          {
            globalOffsetX &&
            lables.map((item,i)=>{

              let topLeft = `${(globalOffsetX + drawOffsetX) * i},${i * 70 + 20}`
              let topRight = `${width - (globalOffsetX + drawOffsetX) * i},${i * 70 + 20}`
              let bottomLeft = `${(globalOffsetX + drawOffsetX) * (i+1)},${(i+1) * 70}`
              let bottomRight = `${width - (globalOffsetX + drawOffsetX) * (i+1)},${(i+1) * 70}`

              let textX = width / 2 - 14 * (item.length / 2);
              let textY = i * 70 + 50;

              return (
                <Fragment key={i}>
                  <polygon fill={colors[i]} points={`${topLeft} ${topRight} ${bottomRight} ${bottomLeft}`} />
                  <text x={textX} y={textY} fill="#fff">{item}</text>
                </Fragment>
              )
            })
          }
        </svg>
        <div
          className={styles.content}
          style={{width: `${width / 2}px`, left: `${width / 2}px`}}
        >
          {
            lables.map((item,i)=>{
              return (
                <p key={i}/>
              )
            })
          }
        </div>

        <div
          className={styles.table}
          style={{
            width: `calc(100% - ${width}px)`,
            left: `${width}px`
          }}
        >
          <div className={styles.bodys} style={{width: _width}}>
            <p >
              {
                heads.map((item,i)=>{
                  return (
                    <span
                      key={i}
                      className={styles.header}
                    >
                    <i>{item}</i>
                  </span>)
                })
              }
            </p>
            {
              bodys.map((item,i)=>{
                let rowType = (i % 2 != 0 && i != 0);

                return (<p key={i}>
                  {
                    item.map((two,index)=>{
                      let color = '#333';
                      if(two.type == 1){
                        color = '#F5222D';
                      }else if(two.type == 2){
                        color = '#52C41A';
                      }
                      if(rowType){
                        return (
                          <span key={index}>
                            {
                              two.value != '' ?
                              (
                                <Tag
                                  value={two.value}
                                  color={color}
                                />
                              ) : ''
                            }
                          </span>
                        )
                      }else{
                        return (
                          <span
                            key={index}
                            style={{color: color}}
                          >{two.value}</span>
                        )
                      }
                    })
                  }
                </p>)
              })
            }
          </div>
        </div>

      </div>
    )
  }
}
export default Funnel;
