import React, { Component, Fragment } from 'react';
import styles from './style.scss';
import Icon from '_c/Icon';
import Proportion from './Proportion';
import Progress from './Progress';
import getPieConfig from './pieConfig'
import EasyChart from '_c/Chart/ChartItem/EasyChart';
import toThousands from './to-thousands';
import { setEvent, triggerEvent } from './cache';

const stepTypes = {
  1: getWord('关注'),
  2: getWord('粗选'),
  3: getWord('兴趣'),
  4: getWord('偏好'),
  5: getWord('意向'),
}

// 国际化处理程序
import { getWord } from 'auto-gjh';

import Loading from '_c/Chart/Loading';

import {
  BorderLeftTop,
  BorderRightTop,
  BorderLeftBottom,
  BorderRightBottom,
  BorderLeftCentent,
  BorderRightCentent
} from './border';

class EthnicHeader extends Component {
  state = {
    preEthnic: [],
    ethnicSelect: [],
    ethnicShow: false,
    ethnicAll: true, // 是否被全部选中
    viewTypes: [getWord('分群查看'), getWord('单元查看')],
    viewIndex: 0,
    errorType: false,
    mask: false
  }

  shouldComponentUpdate(nextProps, nextState) {
    const { changeZuqun } = nextProps;

    if (changeZuqun) {
      return false
    } else {
      return true
    }
  }


  // 初始化族群选择
  componentDidMount() {
    const { tabIndex } = this.props;

    setEvent('setZuqun', () => {

      if (tabIndex === 2) {
        this.selectTop5()
      } else {
        this.selectAll()
      }
    })

    triggerEvent('setZuqun');

  }

  // 选择TOP5
  selectTop5 = () => {

    const { swarmInfo } = this.props;

    const { ethnicList } = swarmInfo;
    let _temp = JSON.parse(JSON.stringify(ethnicList))

    let _ids = _temp.filter(item => item.name !== '全部').filter((item, i) => i < 5).map(item => item.id);
    let _list = _temp.sort((v1, v2) => v1.id - v2.id);

    let preEthnic = [];
    let ethnicSelect = [];

    _ids.forEach(item => {
      let _index = _list.findIndex(_two => _two.id == item);
      preEthnic.push(_index);
      ethnicSelect.push(_index);
    });


    this.setState({
      preEthnic,
      ethnicSelect,
      ethnicAll: false
    }, () => {
      this.ethincOk()
    })

  }

  // 选择所有族群
  selectAll = () => {
    const { swarmInfo } = this.props;

    const { ethnicList } = swarmInfo

    let preEthnic = ethnicList.map((item, i) => i)
    let ethnicSelect = ethnicList.map((item, i) => i)

    this.setState({
      preEthnic,
      ethnicSelect,
      ethnicAll: true
    })
  }

  // 展开族群选择
  setDrop = () => {
    this.setState(state => {
      return {
        ethnicShow: !state.ethnicShow,
        mask: !state.ethnicShow
      }
    });
  }

  // 选择族群
  changeEthnic = (index) => {
    let { ethnicSelect, ethnicAll } = this.state;

    const { swarmInfo } = this.props;
    const { ethnicList } = swarmInfo


    let _index = ethnicSelect.findIndex(item => item == index);
    let ethnicAll1 = false;
    let errorType = false;

    if (index == 0) {
      if (_index != -1) {
        ethnicSelect = []
      } else {
        ethnicSelect = ethnicList.map((item, i) => i);
        ethnicAll1 = true;
      }
    } else {
      if (_index == -1) {
        ethnicSelect.push(index)
        if (ethnicSelect.length >= ethnicList.length - 1) {
          ethnicSelect = ethnicList.map((item, i) => i);
          ethnicAll1 = true;
        }
      } else {
        ethnicSelect.splice(_index, 1);
        if (ethnicAll) {
          let _inx = ethnicSelect.findIndex(item => item == 0);
          ethnicSelect.splice(_inx, 1);
          ethnicAll1 = false;
        }
      }
    }

    if (ethnicSelect.length == 0) {
      errorType = true;
    }


    this.setState({
      errorType,
      ethnicSelect,
      ethnicAll: ethnicAll1
    })
  }

  // 确定族群选择
  ethincOk = () => {
    let { ethnicSelect } = this.state;

    const { swarmInfo, setEthnicData, tabIndex } = this.props;
    const { ethnicList } = swarmInfo

    let preEthnic = JSON.parse(JSON.stringify(ethnicSelect));

    let _ethnicList = JSON.parse(JSON.stringify(ethnicList));

    if (tabIndex === 2) {
      _ethnicList = _ethnicList.sort((v1, v2) => v1.id - v2.id);
    }


    if (preEthnic.length == 0) {
      return
    }

    let ethnicAll = false;

    if (ethnicSelect.length == ethnicList.length) {
      ethnicAll = true
    }

    let _list = preEthnic.map(item => _ethnicList[item])

    this.setState({
      preEthnic,
      ethnicAll,
      ethnicShow: false,
      errorType: false,
      mask: false
    }, () => {
      setEthnicData(_list)
    })
  }

  // 取消族群选择
  ethincCannel = () => {
    let { ethnicSelect, preEthnic } = this.state;
    const { swarmInfo } = this.props;
    const { ethnicList } = swarmInfo

    ethnicSelect = JSON.parse(JSON.stringify(preEthnic));
    let ethnicAll = false;

    if (ethnicSelect.length == ethnicList.length) {
      ethnicAll = true
    }

    this.setState({
      ethnicSelect,
      ethnicAll,
      ethnicShow: false,
      mask: false
    })
  }

  // 切换族群查看方式
  changeTypes = (index) => {

    const { setEthnic, swarmInfo } = this.props;
    const { ethnicList } = swarmInfo;

    this.setState({
      viewIndex: index
    })

    setEthnic(index)
  }

  closeDrop = () => {
    this.setState({
      mask: false,
      ethnicShow: false,
    })

    this.ethincCannel()
  }

  render() {
    const { ethnicSelect, ethnicShow, ethnicAll, viewTypes, viewIndex, errorType, mask } = this.state;
    const { swarmInfo, screenStatus, tabIndex } = this.props;
    const { ethnicList, count, total, value, zqSize } = swarmInfo

    let iconEthnic = ethnicShow ? styles['up'] : styles['drop'];

    let bc1 = screenStatus ? '#0699DF' : '#6486F5';
    let bc2 = screenStatus ? '#C383FF' : '#58BDEF';

    let _ethnicList = JSON.parse(JSON.stringify(ethnicList));

    if (tabIndex === 2) {
      _ethnicList = _ethnicList.sort((v1, v2) => v1.id - v2.id);
    }

    // 用于显示族群的名称
    const showEthnicName = () => {
      if (ethnicAll) return getWord('全部');
      if (ethnicSelect.length > 1) return getWord('多个族群')
      if (ethnicSelect.length == 0) return ''
      return _ethnicList[ethnicSelect[0]].name;
    }

    const getName = () => {
      let _name = tabIndex === 2 ? '标准分群' : '战略分群';
      return _name
    }

    return (
      <div className={styles['info']}>
        {
          mask
          && <div
            className={styles['mask_zhe']}
            onClick={this.closeDrop}
          ></div>
        }
        {/* 左侧 */}
        <div className={styles['info-left']}>
          {
            screenStatus &&
            <Fragment>
              <BorderLeftTop />
              <BorderRightTop />
              <BorderLeftBottom />
              <BorderRightBottom />
            </Fragment>
          }

          {/* 人群单元 */}
          <div className={styles['left-unit']}>
            <p>{getName()}</p>

            {/*  占比显示框  */}
            <div
              className={styles['unit-ethnic']}
              onClick={this.setDrop}
            >
              <span title={showEthnicName()}>{showEthnicName()}</span>
              <Icon
                type='youfanye'
                clm={iconEthnic}
              />
            </div>
          </div>

          <div className={styles['unit-flex']}>
            <Proportion
              title={getWord('人群占比')}
              title_1={getWord('已选用户数')}
              title_2={getWord('总用户数')}
              unit={getWord('人')}
              total={total}
              selected={value}
              waveColor={bc1}
              enStatus
              _width='80px'
            />
          </div>
          <div className={styles['unit-flex-two']}>
            <Proportion
              title={getWord('分群数占比')}
              title_1={getWord(`已选${getName()}数`)}
              title_2={getWord(`总${getName()}数`)}
              unit={getWord('个')}
              total={zqSize}
              selected={count}
              waveColor={bc2}
              enStatus
              _width='110px'
            />
          </div>

          {/*  战略分群选框  */}
          {/*  战略分群选框  */}
          {
            ethnicShow &&
            <div className={styles['ethnic-drop']}>
              {
                screenStatus &&
                <Fragment>
                  <BorderLeftTop />
                  <BorderRightTop />
                  <BorderLeftBottom />
                  <BorderRightBottom />
                </Fragment>
              }
              <div className={styles['drop-pane']}>
                <div className={styles['drop-list']}>
                  {
                    _ethnicList.map((item, i) => {
                      let select = ethnicSelect.includes(i) ? 'xuanze-box ' : 'cancel';
                      let clm = ethnicSelect.includes(i) ? styles['ethnic-active'] : '';

                      const showXuhao = () => {
                        if (tabIndex === 2 && i > 0) {
                          return <span className={styles['drop-xuhao']}>{Number(item.id) + 1}</span>
                        }
                      }

                      const showZhanbi = () => {
                        if (tabIndex === 2 && i > 0) {
                          return <span className={styles['drop-zhanbi']}>{item.rate}%</span>
                        }
                      }
                      return (
                        <div
                          key={`ethnic-${i}`}
                          className={styles['drop-item']}
                          onClick={this.changeEthnic.bind(this, i)}
                        >
                          <Icon type={select} clm={clm} />
                          <p>
                            {showXuhao()}
                            <b title={item.name}>{item.name}</b>
                            {showZhanbi()}
                          </p>
                        </div>
                      );
                    })
                  }
                </div>
                {
                  errorType
                  && <div className={styles['error-tip']}>* {getWord('请至少选择一个')}</div>
                }


                {/* 按钮组 */}
                <div className={styles['drop-btn']}>
                  <div
                    className={styles['drop-ok']}
                    onClick={this.ethincOk}
                  >{getWord('确认')}</div>
                  <div
                    className={styles['drop-cannel']}
                    onClick={this.ethincCannel}
                  >{getWord('取消')}</div>
                </div>
              </div>
            </div>
          }


        </div>

        {/* 右侧 */}
        <div className={styles['info-right']}>

          {
            screenStatus &&
            <Fragment>
              <BorderLeftTop />
              <BorderRightTop />
              <BorderLeftBottom />
              <BorderRightBottom />
            </Fragment>
          }

          {/* 核心人群快速定位 */}
          <div className={styles['right-ethinc']}>
            <p>
              {getWord('查看方式')}
              <span>?</span>
              <b className={styles['right-tip']}>
                <span>
                  {getWord('分群查看：鼠标单击选中分群整体，右侧明细数据为整体描述')}
                </span>
                <span>
                  {getWord('单元查看：鼠标单击选中微分群小块，右侧明细数据为微分群描述')}
                </span>
              </b>
            </p>

            {/*  查看方式类型切换  */}
            <div className={styles['right-ethinc-types']}>
              {
                viewTypes.map((item, i) => {
                  let before = i == 0 ? 'fenqunicon' : 'danyuanicon'
                  let icon = viewIndex == i ? 'danxuan-xuanzhong' : 'danxuan-moren';
                  let clm = viewIndex == i ? styles['view-active'] : styles['view-cannel'];

                  let textColor = viewIndex == i ? styles['text-active'] : ''

                  return (
                    <div
                      className={styles['ethnic-view']}
                      key={`view-${i}`}
                      onClick={this.changeTypes.bind(this, i)}
                    >
                      <Icon type={icon} clm={clm} />
                      <span className={textColor}>{item}</span>
                      <Icon type={before} clm={styles['view-icon']} />
                    </div>
                  )
                })
              }
            </div>
          </div>

        </div>
      </div>
    )
  }
}

class EthnicRight extends Component {

  componentDidMount() {
    if (this.box) {
      this.box.style = 'display: flex'
    }
    setTimeout(() => {
      this.box.style = 'display: none'
    }, 400)
  }

  componentWillUpdate() {
    if (this.box) {
      this.box.style = 'display: flex'
    }
    setTimeout(() => {
      this.box.style = 'display: none'
    }, 600)
  }

  render() {
    const { ethnicInfo, axis, eventType, screenStatus } = this.props;

    const { name, step, rate, value, config, u, v, n, cmpSeries, userValue } = ethnicInfo

    // 显示基础属性
    const showBase = () => {
      if (u && v && n) {
        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('基本属性')}</span>
            </div>

            <div className={styles['ubtitle']}>{axis.uName}</div>

            {
              u.map((item, i) => {
                return (
                  <Progress
                    name={item.name}
                    value={`${item.data}%`}
                    key={`u-${i}`}
                  />
                )
              })
            }

            <div className={styles['ubtitle']}>{axis.vName}</div>

            {
              v.map((item, i) => {
                return (
                  <Progress
                    name={item.name}
                    value={`${item.data}%`}
                    key={`v-${i}`}
                  />
                )
              })
            }

            <div className={styles['ubtitle']}>{axis.nName}</div>

            {
              n.sort((v1, v2) => v2.data - v1.data).map((item, i) => {
                return (
                  <Progress
                    name={item.name}
                    value={`${item.data}%`}
                    key={`n-${i}`}
                  />
                )
              })
            }

          </div>
        )
      }
    }


    // 用户买点
    const showBuy = () => {
      if (config && config.length > 0) {
        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('用户买点')}</span>
            </div>
            {
              config.sort((v1, v2) => v2.data - v1.data).filter((item, i) => i < 3).map((item, i) => {
                return (
                  <Progress
                    key={`config-${i}`}
                    name={`No.${i + 1} ${item.name}`}
                    value={`${item.data}%`}
                  />
                )
              })
            }
          </div>
        )
      }
    }


    // 输出用户阶段
    const showStep = () => {
      if (step && step.length > 0) {
        let _option = getPieConfig();
        let data = step.map(item => {
          return {
            name: stepTypes[item.name],
            y: + item.data
          }
        })
        let _bg = screenStatus ? '#091747' : '#fff';
        let _fontColor = screenStatus ? '#fff' : '#333';

        _option.chart.backgroundColor = _bg;
        _option.plotOptions.pie.dataLabels.formatter = function () {
          let temp = this.point
          let _name = this.point.name;
          return `<span style='font-size: 12px; color: ${_fontColor}'>${_name}</b></span>`
        }
        _option.tooltip = {
          backgroundColor: 'none',
          borderColor: 'none',
          borderRadius: 0,
          shadow: false,
          useHTML: true,
          style: {
            color: "#fff",
            borderRadius: '8px'
          },
          formatter: function () {
            let temp = this.point
            let _name = this.point.name;
            let _value = this.point.y;
            _value = _value.toFixed(2);
            return `<span class='${styles['highcharts-tooltip']}'>${_name}: <b>${_value}%</b></span>`
          }
        }

        _option.series.push({
          name: getWord('用户阶段'),
          data
        })


        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('用户阶段')}</span>
            </div>
            <div className={styles['pie']}>
              <EasyChart
                key={Math.random()}
                option={_option}
                style={{ height: '150px', width: '233px' }}
              />
            </div>
          </div>
        )
      }
    }

    // 输出价值观
    const showJia = () => {
      if (eventType) return false;
      if (userValue && userValue.length > 0) {
        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('价值观')}</span>
            </div>
            {
              userValue.sort((v1, v2) => v2.data - v1.data).map((item, i) => {
                return (
                  <Progress
                    key={`config-${i}`}
                    name={item.name}
                    value={`${item.data}%`}
                  />
                )
              })
            }
          </div>
        )
      }
    }

    // 输出竞争度排行
    const showCmpSeries = () => {
      if (cmpSeries && cmpSeries.length > 0) {
        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('竞争度排行')}</span>
            </div>
            {
              cmpSeries.filter((item, i) => i < 6).map((item, i) => {
                return (
                  <Progress
                    key={`config-${i}`}
                    name={`No.${i + 1} ${item.name}`}
                    value={`${item.data}%`}
                  />
                )
              })
            }
          </div>
        )
      }
    }

    // 显示族群名称
    const showEthnicName = () => {
      let _n = name.split('，');
      return (
        <Fragment>
          <p
            style={{ marginBottom: '10px' }}
            title={_n[0]}
          >{_n[0]}</p>
          <p title={_n[1]}>{_n[1]}</p>
        </Fragment>
      )
    }


    return (
      <div className={styles['pane']}>

        <div className={styles['loading-make']} ref={_b => this.box = _b}>
          <Loading />
        </div>

        {
          screenStatus &&
          <Fragment>
            <BorderLeftTop />
            <BorderRightTop />
            <BorderLeftBottom />
            <BorderRightBottom />
          </Fragment>
        }

        <div className={styles['right-header']}>
          {
            screenStatus &&
            <Fragment>
              <BorderLeftCentent />
              <BorderRightCentent />
            </Fragment>
          }

          {/*  头部标题 */}
          <div className={styles['header-title']} title={name}>
            {showEthnicName()}
          </div>

          {/*  人群单元数 */}
          <div className={styles['header-value']}>
            <span>{getWord('用户数')}</span>
            <span>{getWord('在用户总量中占比')}</span>
          </div>

          {/*  数值  */}
          <div className={styles['header-value']}>
            <span>{toThousands(value)} {getWord('人')}</span>
            <span>{rate}%</span>
          </div>
        </div>

        <div className={styles['right-scroll']}>


          {/*  显示基础属性  */}
          {showBase()}


          {/* 用户买点 */}
          {showBuy()}

          {/** 价值观 */}
          {showJia()}

          {/*  用户阶段  */}
          {!eventType && showStep()}

          {/* 竞争度排行 */}
          {!eventType && showCmpSeries()}
        </div>
      </div>
    )
  }
}

export default {
  Header: EthnicHeader,
  Right: EthnicRight
}
