import React, {Component, Fragment } from 'react';
import styles from './style.scss';
import Icon from '_c/Icon';
import Proportion from './Proportion';
import Progress from './Progress';
import EasyChart from '_c/Chart/ChartItem/EasyChart';
import toThousands from './to-thousands';
import getPieConfig from './pieConfig'

import { setEvent, triggerEvent, clearTargetEvent } from './cache';

// 国际化处理程序
import { getWord } from 'auto-gjh';

import Loading from '_c/Chart/Loading';

import {
  BorderLeftTop,
  BorderRightTop,
  BorderLeftBottom,
  BorderRightBottom,
  BorderLeftCentent,
  BorderRightCentent
} from './border';

const stepTypes = {
  1: getWord('关注'),
  2: getWord('粗选'),
  3: getWord('兴趣'),
  4: getWord('偏好'),
  5: getWord('意向'),
}

class UnitHeader extends Component{
  state = {
    ratoList: [
      '0%','10%','20%','30%','40%','50%',
      '60%','70%','80%','90%','100%'
    ],
    ratoIndex: 7,
    ratoShow: false,
    numList: [1,2,3,4,5,6,7,8,9,10],
    numIndex: 0,
    numShow: false,
    mask: false, // 全屏遮罩
  }

  componentDidMount(){
    setEvent('setTop', (index)=>{
      this.changeNum(index)
    })
  }

  // 点击切换百分比
  changeRato = (index) =>{
    const { setFilter } =this.props;
    const { ratoList } = this.state;
    setFilter(ratoList[index]);
    this.changeNum(0);
    this.setState({
      ratoIndex: index,
      ratoShow: false,
      mask: false
    });
  }

  // 点击切top排名
  changeNum = (index) =>{
    const { setTop } = this.props;
    triggerEvent('setUvnTop', index)
    this.setState({
      numIndex: index,
      numShow: false,
      mask: false
    });
  }

  // 点击展开NO定位
  setNum = (index) =>{
    this.setState(state => {
      return {
        numShow: !state.numShow,
        mask: !state.ratoShow
      }
    });
  }


  // 展开百分比选择
  setDrop =() =>{
    this.setState(state => {
      return {
        ratoShow: !state.ratoShow,
        mask: !state.ratoShow
      }
    });
  }


  closeDrop = () =>{
    this.setState({
      mask: false,
      numShow: false,
      ratoShow: false
    })
  }

  render(){
    const { ratoList, ratoIndex, ratoShow, numList, numIndex, numShow, mask } = this.state;

    const { seriesInfo, screenStatus } = this.props;


    let iconRato = ratoShow ? styles['up'] : styles['drop'];
    let iconNum = numShow ? styles['up'] : styles['drop'];

    let bc1 = screenStatus ? '#0699DF' : '#6486F5';
    let bc2 = screenStatus ? '#C383FF' : '#58BDEF';

    return (
      <div className={styles['info']}>
        {
          mask
          && <div
          className={styles['mask_zhe']}
          onClick={this.closeDrop}
          ></div>
        }

        {/* 左侧 */}
        <div className={styles['info-left']}>

          {
            screenStatus &&
            <Fragment>
              <BorderLeftTop />
              <BorderRightTop />
              <BorderLeftBottom />
              <BorderRightBottom />
            </Fragment>
          }

          {/* 人群单元 */}
          <div className={styles['left-unit']}>
            <p>{getWord('用户数占比')}</p>

            {/*  占比显示框  */}
            <div
              className={styles['unit-rato']}
              onClick={this.setDrop}
            >
              <span>{ratoList[ratoIndex]}</span>
              <Icon
                type='youfanye'
                clm={iconRato}
              />
            </div>
          </div>

          <div className={styles['unit-flex']}>
            <Proportion
              title={getWord('人群占比')}
              title_1={getWord('已选用户数')}
              title_2={getWord('总用户数')}
              unit={getWord('人')}
              total={seriesInfo.total}
              selected={seriesInfo.selectValue}
              waveColor={bc1}
              enStatus
              _width='80px'
             />
          </div>
          <div className={styles['unit-flex-two']}>
            <Proportion
              title={getWord('单元数占比')}
              title_1={getWord('已选块数')}
              title_2={getWord('总块数')}
              unit={getWord('个')}
              total={seriesInfo.smallSize}
              selected={seriesInfo.selectSize}
              waveColor={bc2}
              enStatus
              _width='80px'
            />
          </div>

          {/*  占比选择框  */}
          {
            ratoShow && (
              <div className={styles['unit-rato-drop']}>
                {
                  screenStatus &&
                  <Fragment>
                    <BorderLeftTop />
                    <BorderRightTop />
                    <BorderLeftBottom />
                    <BorderRightBottom />
                  </Fragment>
                }

                <div className={styles['drop-value']}>
                  {
                    ratoList.map((item, i) => {

                      let _y = i == ratoIndex ? styles['yuan-active'] : ''
                      let _t = i == ratoIndex ? styles['text-active'] : ''

                      return (
                        <div
                          className={styles['drop-item']}
                          key={`value-${i}`}
                          onClick={this.changeRato.bind(this, i)}
                        >
                          <div className={`${styles['yuan']} ${_y}`}></div>
                          <p className={_t}>{item}</p>
                        </div>
                      )
                    })
                  }
                </div>
              </div>
            )
          }

        </div>

        {/* 右侧 */}
        <div className={styles['info-right']}>

          {
            screenStatus &&
            <Fragment>
              <BorderLeftTop />
              <BorderRightTop />
              <BorderLeftBottom />
              <BorderRightBottom />
            </Fragment>
          }


          {/* 核心人群快速定位 */}
          <div className={styles['right-unit']}>
            <p>{getWord('核心人群快速定位')}</p>

            {/*  快速定位  */}
            <div
              className={styles['right-rato']}
              onClick={this.setNum}
            >
              <span>No.{numList[numIndex]}</span>
              <Icon
                type='youfanye'
                clm={iconNum}
              />
            </div>
          </div>

          {/*  右侧下拉  */}
          {
            numShow &&
            <div className={styles['unit-num-drop']}>
              {
                screenStatus &&
                <Fragment>
                  <BorderLeftTop />
                  <BorderRightTop />
                  <BorderLeftBottom />
                  <BorderRightBottom />
                </Fragment>
              }

              {
                numList.map((item, i)=>{
                  let _num = i == numIndex ? styles['num-active'] : ''
                  return (
                    <p
                      key={`num-${i}`}
                      onClick={this.changeNum.bind(this,i)}
                      className={_num}
                    >
                      No.{item}
                    </p>
                  )
                })
              }
            </div>
          }

        </div>
      </div>
    )
  }
}

class UnitRight extends Component{

  componentWillUpdate(){
    if(this.box1){
      this.box1.style = 'display: flex'
    }
    setTimeout(()=>{
      if(this.box1){
        this.box1.style = 'display: none'
      }
    }, 600)
  }

  render(){
    const { unitInfo, axis, eventType, screenStatus } = this.props;

    const { name, value, rate, x, y, z, config, step, cmpSeries, userValue } = unitInfo;


    // 输出用户买点
    const showConfig = () => {
      if(config && config.length > 0){
        return (
          <div className={styles['unit-card']}>

            <div className={styles['right-title']}>
              <span>{getWord('用户买点')}</span>
            </div>
            {
              config.sort((v1, v2) => v2.data - v1.data).filter((item,i)=> i < 3).map((item, i)=>{
                return (
                  <Progress
                    key={`config-${i}`}
                    name={`No.${i + 1} ${item.name}`}
                    value={`${item.data}%`}
                  />
                )
              })
            }
          </div>
        )
      }
    }

    // 输出价值观
    const showJia = () => {
      if(eventType) return false;
      if(userValue && userValue.length > 0){
        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('价值观')}</span>
            </div>
            {
              userValue.sort((v1, v2) => v2.data - v1.data).map((item, i)=>{
                return (
                  <Progress
                    key={`config-${i}`}
                    name={`No.${i + 1} ${item.name}`}
                    value={`${item.data}%`}
                  />
                )
              })
            }
          </div>
        )
      }
    }


    // 输出用户阶段
    const showStep = () => {
      if(step && step.length > 0){
        let _option = getPieConfig();
        let data = step.map(item => {
          return {
            name: stepTypes[item.name],
            y: + item.data
          }
        })


        let _bg = screenStatus ? '#091747' : '#fff';
        let _fontColor = screenStatus ? '#fff' : '#333';

        _option.chart.backgroundColor = _bg;
        _option.plotOptions.pie.dataLabels.formatter = function(){
          let temp = this.point
          let _name = this.point.name;
          return `<span style='font-size: 12px; color: ${_fontColor}'>${_name}</b></span>`
        }
        _option.tooltip = {
          backgroundColor: 'none',
          borderColor: 'none',
          borderRadius: 0,
          shadow: false,
          useHTML: true,
          style: {
            color: "#fff",
            borderRadius: '8px'
          },
          formatter: function() {
            let temp = this.point
            let _name = this.point.name;
            let _value = this.point.y;
            _value = _value.toFixed(2);
            return `<span class='${styles['highcharts-tooltip']}'>${_name}: <b>${_value}%</b></span>`
          }
        }

        _option.series.push({
          name: getWord('用户阶段'),
          data
        })


        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('用户阶段')}</span>
            </div>
            <div className={styles['pie']}>
              <EasyChart
                key={Math.random()}
                option={_option}
                style={{height: '150px', width: '233px'}}
              />
            </div>
          </div>
        )
      }
    }

    // 输出竞争度排行
    const showCmpSeries = () => {
      if(cmpSeries && cmpSeries.length > 0){
        return (
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('竞争度排行')}</span>
            </div>
            {
              cmpSeries.filter((item,i)=> i < 6).map((item, i)=>{
                return (
                  <Progress
                    key={`config-${i}`}
                    name={`No.${i+1} ${item.name}`}
                    value={`${item.data}%`}
                  />
                )
              })
            }
          </div>
        )
      }
    }


    return (
      <div className={styles['pane']}>
        <div className={styles['loading-make']} ref={_b => this.box1 = _b}>
          <Loading />
        </div>

        {
          screenStatus &&
          <Fragment>
            <BorderLeftTop />
            <BorderRightTop />
            <BorderLeftBottom />
            <BorderRightBottom />
          </Fragment>
        }

        {/*   头部  */}
        <div className={styles['right-header']}>
          {
            screenStatus &&
            <Fragment>
              <BorderLeftCentent />
              <BorderRightCentent />
            </Fragment>
          }

          {/*  头部标题 */}
          <div className={styles['header-title']}>{name}</div>

          {/*  人群单元数 */}
          <div className={styles['header-value']}>
            <span>{getWord('用户数')}</span>
            <span>{getWord("在用户总量中占比")}</span>
          </div>

          {/*  数值  */}
          <div className={styles['header-value-1']}>
            <span>{toThousands(value)} {getWord('人')}</span>
            <span>{rate}%</span>
          </div>
        </div>


        {/*  滚动区域  */}
        <div className={styles['right-scroll']}>
          {/* 基本属性 */}
          <div className={styles['unit-card']}>
            <div className={styles['right-title']}>
              <span>{getWord('基本属性')}</span>
            </div>

            <div className={styles['unit-base']}>
              <span title={`U-${axis.uName}`}>U-{axis.uName}：</span>
              <span title={z}>{z}</span>
            </div>

            <div className={styles['unit-base']}>
              <span title={`V-${axis.vName}：`}>V-{axis.vName}：</span>
              <span title={y}>{y}</span>
            </div>

            <div className={styles['unit-base']}>
              <span title={`N-${axis.nName}：`}>N-{axis.nName}：</span>
              <span title={x}>{x}</span>
            </div>
          </div>

          {/* 用户买点 */}
          {showConfig()}

          {/** 价值观 */}
          {showJia()}

          {/*  用户阶段  */}
          { !eventType &&showStep() }

          {/* 竞争度排行 */}
          { !eventType && showCmpSeries() }

        </div>
      </div>
    )
  }
}

export default {
  Header: UnitHeader,
  Right: UnitRight
}
