import styles from './style.scss';
import toThousands from './to-thousands';

// 国际化处理程序
import { getWord } from 'auto-gjh';
import { log } from 'util';

const stepName = {
  '1': getWord('关注'),
  '2': getWord('粗选'),
  '3': getWord('兴趣'),
  '4': getWord('偏好'),
  '5': getWord('意向'),
}
// type: true 为单元， false 为族群
// eventType true 为事件 false为车系
function getColumns(type = true, eventType = false, uvnNames = {}, tabType) {

  let columns;

  if (type) {
    columns = [
      {
        title: getWord('序号'),
        dataIndex: 'orderNum',
        align: 'center',
        width: 80,
        fixed: 'left',
      }, {
        title: getWord('人群单元编号'),
        dataIndex: 'id',
        align: 'center',
        width: 150,
        fixed: 'left',
      }, {
        title: getWord('用户总人群包占比'),
        dataIndex: 'rate',
        align: 'center',
        width: 150,
        fixed: 'left',
        render: text => (<span>{text}%</span>),
      }, {
        title: getWord('人群包总量'),
        dataIndex: 'value',
        width: 150,
        fixed: 'left',
        align: 'center',
      },
      {
        title: `U-${getWord(uvnNames.uName)}`,
        dataIndex: 'x',
        width: 150,
        align: 'center',
      },
      {
        title: `V-${getWord(uvnNames.vName)}`,
        dataIndex: 'y',
        width: 150,
        align: 'center',
      },
      {
        title: `N-${getWord(uvnNames.nName)}`,
        dataIndex: 'z',
        width: 150,
        align: 'center',
      },
      {
        title: getWord('用户买点'),
        align: 'center',
        dataIndex: 'config',

        render: (text, record) => (
          <div key='config'>
            {
              text == getWord('暂无')
                ? text
                : text.map((tag, index) => {

                  return (
                    <div className={styles.contain} key={`config-${index}`}>
                      <span
                        style={{
                          maxWidth: '170',
                          overflow: 'hidden',
                          textOverflow: 'ellipsis',
                          whiteSpace: 'nowrap'
                        }}
                        title={record.configName[index]}
                        className={styles.name}
                      >

                        {record.configName[index]}

                      </span>

                      <div style={{ width: '100%' }}>

                        <span
                          className={styles.progress}
                          style={{ width: tag + '%' }}>
                        </span>

                      </div>
                      <span className={styles.data}>{tag + '%'}</span>
                    </div>
                  )
                })
            }
          </div>
        ),
      },
      {
        title: getWord('价值观'),
        dataIndex: 'userValue',
        align: 'center',
        width: 350,
        render: (text, record) => {

          if (text instanceof Array) {
            if (text.length === 0) {
              return (<div key='userValue'>{getWord('暂无')}</div>)
            }
          }
          
          return (
            <div key='userValue'>
              {
                text == getWord('暂无')
                  ? text
                  : text.map((tag, index) => {

                    return (
                      <div className={styles.contain} key={`userValue-${index}`}>
                        <span className={styles.name}>
                          {tag.name}
                        </span>

                        <div style={{ width: '100%' }}>
                          <span
                            className={styles.progress}
                            style={{ width: tag.data + '%' }}
                          >
                          </span>
                        </div>

                        <span className={styles.data}>
                          {tag.data + '%'}
                        </span>
                      </div>
                    )
                  })}
            </div>
          )
        },
      },
      {
        title: getWord('用户阶段'),
        dataIndex: 'step',
        align: 'center',
        width: 350,
        render: (text, record) => (
          <div key='step'>
            {
              text == getWord('暂无')
                ? text
                : text.map((tag, index) => {

                  return (
                    <div className={styles.contain} key={`step-${index}`}>
                      <span className={styles.name}>
                        {record.stepName[index]}
                      </span>

                      <div style={{ width: '100%' }}>
                        <span
                          className={styles.progress}
                          style={{ width: tag + '%' }}
                        >
                        </span>
                      </div>

                      <span className={styles.data}>
                        {tag + '%'}
                      </span>
                    </div>
                  )
                })}
          </div>
        ),
      },
      {
        title: getWord('竞争度排行'),
        dataIndex: 'cmpSeries',
        align: 'center',
        width: 350,
        render: (text, record) => (
          <div key='cmpSeries'>
            {
              text == getWord('暂无')
                ? text
                : text.map((tag, index) => {

                  return (
                    <div className={styles.contain} key={`cmpSeries-${index}`}>
                      <span
                        className={styles.name}
                        style={{
                          maxWidth: '170',
                          overflow: 'hidden',
                          textOverflow: 'ellipsis',
                          whiteSpace: 'nowrap'
                        }}
                        title={tag.name}
                      >
                        {tag.name}
                      </span>

                      <div style={{ width: '100%' }}>
                        <span
                          className={styles.progress}
                          style={{ width: tag.data + '%' }}
                        >
                        </span>
                      </div>

                      <span className={styles.data}>
                        {tag.data + '%'}
                      </span>
                    </div>
                  )
                })}
          </div>
        ),
      }
    ];

    if (eventType) {
      columns = columns.filter(item => item.dataIndex != 'userValue')
    }
    if (eventType) {
      columns = columns.filter(item => item.dataIndex != 'step').filter(item => item.dataIndex != 'cmpSeries');
    }
  } else {
    columns = [{
      title: getWord('族群名称'),
      width: 250,
      dataIndex: 'name',
      align: 'center',
      fixed: 'left',
      render: text => {
        if (!text) return ''
        let name = text.split('，')
        return (<div>
          <p>{name[0]}</p>
          <p>{name[1]}</p>
        </div>)
      }
    }, {
      title: getWord('人群占比'),
      width: 150,
      dataIndex: 'rate',
      align: 'center',
      fixed: 'left',
      sorter: (a, b) => a.rate - b.rate,
      defaultSortOrder: 'descend',
      render: text => (<span>{text}%</span>),
    }, {
      title: getWord('人群总量'),
      width: 150,
      dataIndex: 'value',
      align: 'center',
      fixed: 'left',
      sorter: (a, b) => a.value - b.value,
      render: text => (<span>{toThousands(text)}</span>),
    },
    {
      title: getWord('微分群块数'),
      width: 150,
      dataIndex: 'childNum',
      align: 'center',
      sorter: (a, b) => a.childNum - b.childNum,
      render: text => (<span>{toThousands(text)}</span>),
    },
    {
      title: getWord('块均人群占比'),
      width: 150,
      dataIndex: 'childRate',
      align: 'center',
      sorter: (a, b) => a.childRate - b.childRate,
      render: text => (<span>{text}%</span>),
    },
    {
      title: `U-${uvnNames.uName}`,
      dataIndex: 'rich',
      align: 'center',
      render: formatCell_2
    },
    {
      title: `V-${uvnNames.vName}`,
      width: 350,
      dataIndex: 'city',
      align: 'center',
      render: formatCell_no_1
    },
    {
      title: `N-${uvnNames.nName}`,
      width: 350,
      dataIndex: 'need',
      align: 'center',
      render: formatCell_2
    },
    {
      title: getWord('用户买点'),
      width: 350,
      align: 'center',
      dataIndex: 'config',
      render: formatCell
    },
    {
      title: getWord('价值观'),
      width: 350,
      dataIndex: 'userValue',
      align: 'center',
      render: formatCell
    },
    {
      title: getWord('用户阶段'),
      width: 350,
      dataIndex: 'step',
      align: 'center',
      render: formatCell_1
    },
    {
      title: getWord('竞争度排行'),
      width: 350,
      dataIndex: 'cmpSeries',
      align: 'center',
      render: formatCell
    }
    ];


    if (eventType) {
      columns = columns.filter(item => {
        if (item.dataIndex == 'userValue'
          || item.dataIndex == 'childNum'
          || item.dataIndex == 'childRate') {
          return false
        } else {
          return true
        }
      })
    }

    if (eventType) {
      columns = columns.filter(item => item.dataIndex != 'step').filter(item => item.dataIndex != 'cmpSeries');
    }
  }
  return columns
}

// seriesType: true 本品， false: 竞品
// zuqunType: true 单元， false: 族群
// eventType true 事件模式  fasle 车系模式
//
function getData(
  seriesType = true,
  zuqunType = true,
  options,
  eventType = false
) {
  let _series;
  let _data;

  if (seriesType) {
    _series = options.seriesData
  } else {
    _series = options.competData
  }


  if (zuqunType) {
    _data = formatData(_series.square, eventType)
  } else {
    _data = formatData1(_series.zqSquares, eventType)
  }

  return JSON.parse(JSON.stringify(_data))
}


const formatCell = (text) => {

  let _temp = [];
  if (text) {
    _temp = text
  }

  _temp = _.sortBy(_temp, function (item) {
    return -item.data;
  })
  _temp = _temp.filter((item, i) => i < 6);

  let order = Math.floor(Math.random() * 10000);

  return (
    <div >
      {
        _temp.length == 0
          ? getWord('暂无')
          : _temp.map((tag, index) => {
            return (
              <div
                className={styles.contain}
                key={`${order}-${index}`}
              >
                <span
                  style={{
                    maxWidth: '190',
                    overflow: 'hidden',
                    textOverflow: 'ellipsis',
                    whiteSpace: 'nowrap'
                  }}
                  title={tag.name}
                  className={styles.name1}
                >
                  {tag.name}
                </span>

                <div style={{ width: '100%' }}>
                  <span
                    className={styles.progress}
                    style={{ width: tag.data + '%' }}>
                  </span>
                </div>


                <span className={styles.data}>{tag.data + '%'}</span>
              </div>
            )
          })}
    </div>
  )
}

const formatCell_no = (text) => {

  let _temp = [];
  if (text) {
    _temp = text
  }

  _temp = _temp.filter((item, i) => i < 6);

  let order = Math.floor(Math.random() * 10000);

  return (
    <div >
      {
        _temp.length == 0
          ? getWord('暂无')
          : _temp.map((tag, index) => {
            return (
              <div
                className={styles.contain}
                key={`${order}-${index}`}
              >

                <span className={styles.name}>{tag.name}</span>

                <div style={{ width: '100%' }}>
                  <span
                    className={styles.progress}
                    style={{ width: tag.data + '%' }}
                  ></span>
                </div>

                <span className={styles.data}>{tag.data + '%'}</span>
              </div>
            )
          })
      }
    </div>
  )
}

const formatCell_no_1 = (text) => {

  let _temp = [];
  if (text) {
    _temp = text
  }

  _temp = _temp.filter((item, i) => i < 6);

  let order = Math.floor(Math.random() * 10000);

  return (
    <div>
      {
        _temp.length == 0
          ? getWord('暂无')
          : _temp.map((tag, index) => {
            return (
              <div
                className={styles.contain}
                key={`${order}-${index}`}
              >
                <span className={styles.name0}>{tag.name}</span>

                <div
                  style={{ width: '100%' }}>
                  <span
                    className={styles.progress}
                    style={{ width: tag.data + '%' }}>
                  </span>
                </div>

                <span className={styles.data}>{tag.data + '%'}</span>
              </div>
            )
          })
      }
    </div>
  )
}
// 格式化数据
const formatCell_1 = (text) => {

  let _temp = [];
  if (text) {
    _temp = text
  }
  _temp = _temp.filter((item, i) => i < 5);

  let order = Math.floor(Math.random() * 10000);

  return (
    <div>
      {
        _temp.length == 0
          ? getWord('暂无')
          : _temp.map((tag, index) => {
            return (
              <div
                className={styles.contain}
                key={`${order}-${index}`}
              >

                <span
                  style={{
                    maxWidth: '170',
                    overflow: 'hidden',
                    textOverflow: 'ellipsis',
                    whiteSpace: 'nowrap'
                  }}
                  title={stepName[tag.name]}
                  className={styles.name}
                >
                  {stepName[tag.name]}
                </span>

                <div style={{ width: '100%' }}>
                  <span
                    className={styles.progress}
                    style={{ width: tag.data + '%' }}>
                  </span>
                </div>

                <span className={styles.data}>{tag.data + '%'}</span>

              </div>
            )
          })}
    </div>
  )
}

// 格式化数据
const formatCell_2 = (text) => {

  let _temp = [];
  if (text) {
    _temp = text
  }

  _temp = _.sortBy(_temp, function (item) {
    return -item.data;
  })
  _temp = _temp.filter((item, i) => i < 5);

  let order = Math.floor(Math.random() * 10000);

  return (
    <div>
      {
        _temp.length == 0
          ? getWord('暂无')
          : _temp.map((tag, index) => {
            return (
              <div
                className={styles.contain}
                key={`${order}-${index}`}
              >
                <span
                  style={{
                    maxWidth: '190',
                    overflow: 'hidden',
                    textOverflow: 'ellipsis',
                    whiteSpace: 'nowrap'
                  }}
                  title={tag.name}
                  className={styles.name1}
                >
                  {tag.name}
                </span>


                <div style={{ width: '100%' }}>
                  <span
                    className={styles.progress}
                    style={{ width: tag.data + '%' }}>
                  </span>
                </div>

                <span className={styles.data}>{tag.data + '%'}</span>
              </div>
            )
          })}
    </div>
  )
}


//处理表格数据
const formatData = (data, eventType) => {
  const arr = [];

  let _data = JSON.parse(JSON.stringify(data))

  _data.forEach((item, index) => {

    //临时，去掉其他选项
    if (!(item.config === null)) {
      item.config = item.config.filter((item) => {
        return item.name != getWord('其他')
      })
    }

    let _arr = {
      orderNum: index + 1,
      id: item.id === null ? getWord('暂无') : item.id,
      rate: item.rate === null ? getWord('暂无') : item.rate,
      value: item.value === null ? getWord('暂无') : toThousands(item.value),
      x: item.x === null ? getWord('暂无') : item.x,
      y: item.y === null ? getWord('暂无') : item.y,
      z: item.z === null ? getWord('暂无') : item.z,
      config: item.config === null ? getWord('暂无') : [],
      userValue: item.userValue ? [] : getWord('暂无'),
      cmpSeries: item.cmpSeries ? item.cmpSeries : getWord('暂无'),
      configName: [],
      step: item.step === null ? getWord('暂无') : [],
      stepName: [],
    }

    if (item.userValue) {
      item.userValue.map((_item, _index) => {
        _arr.userValue.push(item.userValue[_index]);
      })
      _arr.userValue = _arr.userValue.sort((v1, v2) => v2.data - v1.data);
    }

    if (!eventType && item.step) {
      item.step.map((_item, _index) => {
        _arr.step.push(_item.data);
        _arr.stepName.push(stepName[_item.name]);
      })
    }

    if (item.config) {
      item.config.map((_item, _index) => {
        if (_index < 5) {
          _arr.config.push(item.config[_index].data);
          _arr.configName.push(item.config[_index].name);
        }
      })
    }

    arr.push(_arr)
  })

  return arr
}


// 格式化族群数据
const formatData1 = (data) => {
  let temp = JSON.parse(JSON.stringify(data));
  let _nArr = [];
  for (let x in temp) {
    _nArr.push(temp[x]);
  }
  _nArr = _.sortBy(_nArr, function (item) {
    return -item.value;
  });

  _nArr = _nArr.map(item => {

    let { u, v, n } = item;

    return {
      ...item,
      rich: u,
      city: v,
      need: n,
      orderNum: item.id
    }
  })

  return _nArr
}

export {
  getColumns,
  getData
}
