import React, { Component, Fragment } from 'react';
import styles from './style.scss';
import CreatUVNB from '3d-a';
import Icon from '_c/Icon';
import Unit from './Unit';
import Ethnic from './Ethnic';
import Tables from './Tables';
import Loading from '_c/Chart/Loading';

import { getColumns, getData } from './format';

import { setEvent, triggerEvent } from './cache';

// 事件处理程序
import { bindEvent, clearEvent, dispatchEvent } from './event';

import EasyChart from '_c/Chart/ChartItem/EasyChart';

// 国际化处理程序
import { getWord } from 'auto-gjh';
import { log } from 'util';

class Uvn extends Component {
  state = {
    tabs: [getWord('微分群'), getWord('战略分群'), '标准分群'],
    seriesCompet: [], // 本竞品数组
    tabIndex: 0, // tab 索引, 0: 单元  1:族群
    seriesInfo: {}, //
    swarmInfo: {}, // 分群数据
    unitInfo: {}, // 单元的详细数据
    ethnicInfo: {}, // 族群的详细数据
    ethnicType: 0, // 0：分群查看  1：单元查看
    toggleAcitve: false,
    autoplayIndex: 0, // 0: 本品  1: 竞品  2: 自动播放
    toggleIndex: 0, // 自动切换时的值
    toggleType: false, // 自动切换的状态是否开启
    tableOption: {}, // 表格数据
    changeStatus: true, // 切换时的状态
    bigStatus: false, // 是否为轴切换状态 true: 切轴 false :不切
    screenStatus: false, // true 全屏状态  false 白屏状态
    pageLoad: false, // 页面内切换
    changeZuqun: false,
    error: false
  }

  componentDidCatch(err, inof) {
    this.setState({ error: true })
  }

  componentDidMount() {
    this.preTop = 0;
    this.errorState = false;

    const { options } = this.props;

    if (options.compet) {
      this.setState({
        seriesCompet: options.seriesCompet,
        toggleAcitve: true
      })
    }

    // 当前处于战略分群
    this.pageLoad = true;
    // 上次的标签状态
    this.nextPageLoad = true;

    this.init(options)

  }

  // 加载模型
  init(options) {
    const { tableShow } = this.props;

    try {
      if (!options.seriesData) {
        throw '暂无数据'
      }

      this.uvn = new CreatUVNB(
        this._box,
        options,
        this.clickRect,
        options.type
      )

      // 3D图初始化
      this.uvn.init()
      this.setFilter('70%');
      this.uvn.draw()

      // 表格初始化
      if (!tableShow) {
        this.setTablesData()
      }

      setEvent('setUvnTop', (index) => {
        this.uvn.setTop(index);
      })


      let obj = document.getElementById('mainBox');

      bindEvent(obj, 'scroll', (e) => {
        let top = obj.scrollTop;
        this.preTop = top;
        this.uvn.setScrollOffset(top);
      })


      this.errorState = false;

    } catch (err) {
      this.errorState = true;
      this.setState({ error: true })
    }

    // 屏幕改变事件
    bindEvent(window, 'resize', (event) => {

      event.preventDefault();
      event.stopPropagation();

      // 判断是否全屏
      let isFullscreen = document.fullScreen || document.mozFullScreen || document.webkitIsFullScreen;

      this.setState(() => ({ screenStatus: isFullscreen }), () => {
        if (this.uvn) {
          this.uvn.changePageSize(isFullscreen);

          setTimeout(() => {
            let el = document.getElementById('uvn-box');
            this.uvn.setScreenSize(el);
            this.uvn.onWindowResize();
            if (isFullscreen) {
              this.uvn.setScrollOffset(0);
            } else {
              this.uvn.setScrollOffset(this.preTop);
            }
          }, 100)
        }
      })
    })

  }

  // 销毁模型
  xiaohui() {
    clearEvent()
    if (this.uvn) {
      this.uvn.cleanEvent();
      this.uvn = null;
    }

  }

  // 卸载绑定的事件
  componentWillUnmount() {
    this.xiaohui();
  }

  // 切换人群单元和族群
  setTab = (index) => {
    // 如果点击的是当前标签，则不做反应
    if (index === this.state.tabIndex) return;

    switch (index) {
      case 1:
        this.nextPageLoad = true;
        break;
      case 2:
        this.nextPageLoad = false;
        break;
    }

    this.setState({
      tabIndex: index,
      autoplayIndex: 0,
      ethnicType: 0,
      toggleType: false,
      filterValue: '70%',
      changeStatus: false,
      error: false
    }, () => {

      // 是否有定时器
      if (this.timer) {
        // 关闭自动播放
        this.uvn.autopaly = false;

        window.clearInterval(this.timer);

        this.timer = null;
      }


      this.changeTabs(index);

    })
  }

  // 切换
  async changeTabs(index) {
    const { options, eventType } = this.props;

    let {
      seriesData,
      seriesBiaoData,
      competData,
      competBiaoData,
      ...other
    } = options;


    let _option;

    try {
      if (this.nextPageLoad) {
        _option = {
          seriesData: seriesData,
          competData: competData,
          ...other
        }
      } else {
        if (seriesBiaoData.zqSquares.length === 0) {
          throw '没有数据'
        }
        if (competBiaoData.zqSquares.length === 0) {
          throw '没有数据'
        }

        _option = {
          seriesData: seriesBiaoData,
          competData: competBiaoData,
          ...other
        }
      }
    } catch (err) {
      this.setState({ error: true })
    }


    if (this.pageLoad === this.nextPageLoad) {

      if (this.errorState) {
        let _s = seriesData ? seriesData : seriesBiaoData;
        let _c = competData ? competData : competBiaoData;

        this.nextPageLoad = seriesData ? true : false;

        _option = {
          seriesData: _s,
          competData: _c,
          ...other
        }
        this.init(_option)
      }

      if (!this.errorState) {
        // 切换 单元查看还是族群查看
        this.uvn.zuqunType = index;

        this.uvn.ethniclist = null;

        // 获取数据
        this.setAutoplay(0)
      }

    } else {

      // 销毁UVN图形
      this.xiaohui()

      if (this._box) {
        this._box.innerHTML = '';
      }


      this.setState(state => ({ pageLoad: true }))

      this.init(_option);

      this.setState(state => ({ pageLoad: false }))

      if (!this.errorState) {
        // 切换 单元查看还是族群查看
        this.uvn.zuqunType = index;

        this.uvn.ethniclist = null;

        // 获取数据
        this.setAutoplay(0)
      }

      this.pageLoad = this.nextPageLoad;
    }

  }

  // 设置族群查看方式
  // 0:分群查看
  // 1:单元查看
  setEthnic = (index) => {

    const { ethniclist } = this.state;

    this.setState({ ethnicType: index }, () => {

      this.uvn.changeEthnic(index);

      if (this.state.bigStatus) {
        // 变为不切轴的状态
        this.setBig(false);
      }

      let info = this.uvn.getEthnicData(ethniclist);


      if (index == 1) {
        this.uvn.setTopZuqun(info.maxIndex)
      } else {
        this.uvn.setTopZuqun(info.maxIndex, true)
      }
    })

    setTimeout(() => {
      window.dispatchEvent(new Event('resize'));
    }, 0)

  }

  // 切换本竞品及自动切换
  // 0: 本品
  // 1: 竞品
  // 2: 自动播放
  setAutoplay = (index) => {
    const { tabIndex } = this.state;

    let toggleType = index == 2 ? true : false

    this.setState(() => ({
      autoplayIndex: index,
      toggleType,
      ethniclist: null
    }))


    if (tabIndex != 0 && index != 2) {
      this._box.style.display = 'none';
      this.setState(() => ({ changeZuqun: true }))
      setTimeout(() => {
        triggerEvent('setZuqun');
        this._box.style.display = 'block';
        this.setState(() => ({ changeZuqun: false }))
      }, 0)
    }

    if (this.state.bigStatus) {
      // 变为不切轴的状态
      this.setBig(false);
    }


    if (index == 2) {

      // 开启自动切换
      this.autoPlayInit();
      this.uvn.autopaly = true;
      return

    } else {

      // 关闭自动切换
      if (this.timer) {
        window.clearInterval(this.timer);
        this.timer = null;
      }


      this.setState(
        { toggleIndex: index },
        () => {

          this.uvn.autopaly = false;

          this.setChangeType();

        }
      )

    }

  }

  // 自动播放
  autoPlayInit = () => {


    this.timer = setInterval(() => {

      this.setState(state => {
        const { toggleIndex } = state;

        let _toggleIndex = 1 - toggleIndex;

        return { toggleIndex: _toggleIndex }

      }, () => {
        this.setChangeType()
      })

    }, 3000)

  }


  // 定义切换的类型
  setChangeType = () => {

    // tabIndex : 0:单元  1:族群
    // autoplayIndex   0: 本品  1:竞品
    // filterValue  之前设置的过滤值
    // ethnicType 0: 分群查看  1:单元查看
    const { tabIndex, toggleIndex, filterValue, ethnicType, toggleType } = this.state;

    const { tableShow } = this.props;

    // 更改实例的本竞品类型
    this.uvn.seriesType = toggleIndex == 0;

    if (this.state.bigStatus) {
      // 变为不切轴的状态
      this.setBig(false);
    }

    // 变更本竞品显示
    this.uvn.autoPlay();

    switch (tabIndex) {
      case 0:
        this.setFilter(filterValue);
        triggerEvent('setTop', 0)
        break;
      case 1:
        this.setEthnic(ethnicType);
        this.setEthnicData();
        break;
      case 2:
        if (toggleType) {
          setTimeout(() => {
            this.setEthnicData();
            triggerEvent('setZuqun');
          }, 0);

        } else {
          this.setEthnic(ethnicType);
          this.setEthnicData();
        }
        break;
    }


    if (!tableShow) {
      this.setTablesData()
    }

    // 标记切换逻辑
    this.setState({ changeStatus: true })
  }


  // 获取小方格详细数据
  clickRect = (info, type) => {
    if (type) {
      this.setState({ unitInfo: info })
    } else {
      this.setState({ ethnicInfo: info })
    }
  }


  // 获取族群数据
  setEthnicData = (views) => {
    const { ethnicType } = this.state;

    if (this.state.bigStatus) {
      // 变为不切轴的状态
      this.setBig(false);
    }

    let info = this.uvn.getEthnicData(views);

    if (ethnicType == 0) {
      this.uvn.setTopZuqun(info.maxIndex, true)
    } else {
      this.uvn.setTopZuqun(info.maxIndex, false)
    }

    this.setState(() => ({ swarmInfo: info, ethniclist: views }))
  }


  // 设置过滤值
  setFilter = (value) => {
    let info = this.uvn.setSeriesFilter(value);

    if (this.state.bigStatus) {
      // 变为不切轴的状态
      this.setBig(false);
    }

    this.setState({
      seriesInfo: info,
      filterValue: value
    })
  }

  // 设置表格数据
  setTablesData = () => {
    const { tabs, tabIndex, autoplayIndex } = this.state;

    const { eventType, uvnNames, options, tabType } = this.props;

    let {
      seriesData,
      seriesBiaoData,
      competData,
      competBiaoData,
      ...other
    } = options;

    let _option;

    if (this.nextPageLoad) {
      _option = {
        seriesData: seriesData,
        competData: competData,
        ...other
      }
    } else {
      _option = {
        seriesData: seriesBiaoData,
        competData: competBiaoData,
        ...other
      }
    }

    // 如果是自动切换就跳出
    if (autoplayIndex == 2) return;

    let _name = tabs[tabIndex];
    let _series = options.seriesCompet[autoplayIndex];

    let _seriesType = autoplayIndex == 0;

    let type = tabIndex == 0;

    let _column = getColumns(type, eventType, uvnNames, tabType);

    let _data = getData(_seriesType, type, _option);

    let tableOption = {
      name: `${_name}/${_series}`,
      columns: _column,
      data: _data,
      seriesType: type,
      eventType: eventType
    }

    this.setState({ tableOption })
  }

  // 放大处理
  setBig = (status) => {

    const { tabIndex, ethnicType, swarmInfo, autoplayIndex } = this.state;

    if (autoplayIndex == 2) return

    this.setState({ bigStatus: status })

    this.uvn.setBig(status);

    if (tabIndex !== 0) {
      if (ethnicType == 0) {
        this.uvn.setTopZuqun(swarmInfo.maxIndex, true)
      } else {
        this.uvn.setTopZuqun(swarmInfo.maxIndex, false)
      }
    } else {
      this.uvn.setTop(0, false)
    }
  }

  // 设置全屏状态
  setScreen = (value) => {
    let el = this.screenBox;
    if (value) {//进入全屏
      (el.requestFullscreen && el.requestFullscreen()) ||
        (el.mozRequestFullScreen && el.mozRequestFullScreen()) ||
        (el.webkitRequestFullscreen && el.webkitRequestFullscreen()) ||
        (el.msRequestFullscreen && el.msRequestFullscreen());
    } else { //退出全屏
      document.exitFullscreen ? document.exitFullscreen() :
        document.mozCancelFullScreen ? document.mozCancelFullScreen() :
          document.webkitExitFullscreen ? document.webkitExitFullscreen() : '';
    }
  }

  // 下载Excel
  downExcel = () => {
    const { downLoad } = this.props;
    const { toggleIndex } = this.state;
    downLoad(toggleIndex)
  }

  // 获取标准族群趋势
  getBiaozhun = () => {
    const { showFenqun } = this.props;
    const { ethniclist, swarmInfo, autoplayIndex } = this.state;
    let _list = ethniclist ? ethniclist : swarmInfo.ethnicList;

    _list = _list.filter(item => item.name !== '全部').map(item => item.id)
    _list = _list.join(',')
    showFenqun(_list, autoplayIndex)
  }

  render() {
    const {
      tabs,
      tabIndex,
      seriesInfo,
      ethnicType,
      ethnicInfo,
      unitInfo,
      swarmInfo,
      seriesCompet,
      toggleAcitve,
      autoplayIndex,
      tableOption,
      changeStatus,
      bigStatus,
      screenStatus,
      toggleIndex,
      pageLoad,
      changeZuqun,
      error
    } = this.state;

    var lang = localStorage.getItem("lang");

    let bigIcon = bigStatus ? 'suoxiao' : 'fangda';

    let screenIcon = screenStatus ? 'tuichuquanping' : 'fullscreen'

    let _style = screenStatus ? styles['big'] : styles['small'];

    let gjh_style = lang == 'zh' ? '' : styles['gjh'];

    const {
      options,
      uvnNames,
      eventType,
      tableShow,
      tabType,
      visible,
      handleCancel,
      lineChart
    } = this.props;

    // 格式化趋势弹出层 —— 判断是否位大屏
    if (screenStatus && visible && lineChart) {

      lineChart.chart.plotBackgroundColor = '#091747';

      lineChart.chart.backgroundColor = '#091747';

      lineChart.yAxis.gridLineColor = '#263056';

      lineChart.xAxis.labels = {
        style: {
          textOverflow: 'none',
          whiteSpace: 'nowrap',
          color: '#fff',
          fontSize: '14px',
        }
      }
      lineChart.yAxis.labels = {
        style: {
          textOverflow: 'none',
          whiteSpace: 'nowrap',
          color: '#fff',
          fontSize: '14px',
        }
      }
      lineChart.legend.itemStyle = {
        color: '#fff',
        fontSize: '14px',
      }
      lineChart.legend.itemHoverStyle = {
        color: '#d7d7d7'
      }
    }

    return (
      <div className={styles['car-uvn-box']}>


        <div className={styles['uvn-control']}>

          <div
            className={`${_style} ${gjh_style}`}
            ref={_screenBox => this.screenBox = _screenBox}
          >

            {/*  *趋势弹出层  */}
            {
              visible &&
              <div className={styles['mask']}>
                <div className={styles['mask-body']}>
                  <div
                    onClick={handleCancel}
                    className={styles['mask-close']}>
                    <Icon type='chenghao' />
                  </div>
                  <EasyChart
                    option={lineChart}
                    style={{ height: '450px' }}
                  />
                </div>
              </div>
            }

            {/*  加载 */}
            {
              pageLoad &&
              <div className={styles['loading-page']}>
                <Loading />
              </div>
            }

            {/* UVN头部标签切换 及全屏控制 */}
            <div className={styles['uvn-header']}>

              {/*  标签切换  */}
              <div className={styles['uvn-tabs']}>
                {
                  tabs.map((item, i) => {
                    let cls = tabIndex == i ? styles['tab-active'] : '';

                    // 判断是否加载标准分群标签
                    if (i === 2 && !tabType) return;

                    return <p
                      key={`tab-${i}`}
                      className={cls}
                      onClick={this.setTab.bind(this, i)}
                    >{item}</p>
                  })
                }
              </div>

              {/*  全屏控制  */}
              <div
                className={styles['tab-scree']}
                onClick={this.setScreen.bind(this, !screenStatus)}
              >
                <Icon type={screenIcon} />
              </div>
            </div>


            {
              error ?
                <div className={styles['error-box']}>
                  <p>暂无数据</p>
                </div>
                :
                <Fragment>
                  {/* 头部面板 */}
                  {
                    changeStatus &&
                    (tabIndex == 0
                      ? <Unit.Header
                        setFilter={this.setFilter}
                        seriesInfo={seriesInfo}
                        screenStatus={screenStatus}
                      />
                      : <Ethnic.Header
                        swarmInfo={swarmInfo}
                        setEthnic={this.setEthnic}
                        setEthnicData={this.setEthnicData}
                        screenStatus={screenStatus}
                        tabIndex={tabIndex}
                        changeZuqun={changeZuqun}
                      />)
                  }

                  {/* 3D主体以及右边 */}
                  <div className={styles['body']}>

                    {/* 中心主体 */}
                    <div className={styles['three-3d']}>
                      {/* */}
                      <div className={styles['zhuti-box']}>
                        {
                          changeZuqun &&
                          <div className={styles['zhuti-box-loading']}>
                            <Loading />
                          </div>
                        }
                        <div
                          id='uvn-box'
                          className={styles['uvn-box']}
                          ref={box => this._box = box}
                        />
                      </div>

                      {/*   轴切换  */}
                      <div
                        className={styles['setAsix']}
                        onClick={this.setBig.bind(this, !bigStatus)}
                        title='放大'
                      >
                        <Icon type={bigIcon} />
                      </div>

                      {/**标准分群趋势 */}
                      {
                        tabIndex == 2 && autoplayIndex !== 2 &&
                        <div
                          onClick={this.getBiaozhun}
                          className={styles['biaozhun']}
                          title='标准分群趋势'
                        >
                          <Icon type='zoushi' />
                        </div>
                      }


                      {/* 本竞品 */}
                      {
                        toggleAcitve &&
                        <div className={styles['compet']}>
                          {
                            seriesCompet.map((item, i) => {
                              let _icon = autoplayIndex == i ? 'danxuan-xuanzhong' : 'danxuan-moren';
                              let cls = autoplayIndex == i ? styles['radio-active'] : styles['radio'];

                              let _color = ''
                              if (autoplayIndex == 2) {
                                _color = i == toggleIndex ? '#4481FF' : ''
                              }

                              return (
                                <div
                                  key={`play-${i}`}
                                  className={styles['radio-item']}
                                  onClick={this.setAutoplay.bind(this, i)}
                                >
                                  <Icon type={_icon} clm={cls} />
                                  <span style={{ color: _color }}>{item}</span>
                                </div>
                              )
                            })
                          }
                        </div>
                      }

                    </div>


                    {/*  右侧面板 */}
                    {
                      changeStatus &&
                      (
                        (tabIndex == 0 || ethnicType == 1)
                          ? <Unit.Right
                            unitInfo={unitInfo}
                            axis={uvnNames}
                            eventType={eventType}
                            screenStatus={screenStatus}
                          />
                          : <Ethnic.Right
                            ethnicInfo={ethnicInfo}
                            axis={uvnNames}
                            eventType={eventType}
                            screenStatus={screenStatus}
                          />
                      )
                    }
                  </div>
                </Fragment>
            }

          </div>

          {
            (!tableShow && !pageLoad && !error)
            && <div className={styles['uvn-table']}>
              <Tables {...tableOption} downExcel={this.downExcel} />
            </div>
          }

        </div>
      </div>
    )
  }
}
export default Uvn;
