const _base = {
  colors: [
    '#1CA1FF',
    '#FB9500',
    '#A7DE02',
    '#36D5A8',
    '#FF5F5F',
    '#E501FF',
    '#6C01FF',
    '#3436C7',
    '#B7C0FF',
    '#F90D6C',
    '#31D2FF',
    '#DC0C00'
  ],
  title: {
    text: null,
  },
  credits: {
    enabled: false,
  },
  exporting: {
    enabled: false,
  },
  legend: {
    enabled: false,
    itemStyle: {
      color: '#666',
    },
  },
  xAxis: {
    crosshair: true,
    categories: [],
    min: 0,
    title: {
      text: null,
    },
  },
  yAxis: {
    min: 0,
    title: {
      text: null,
    },
  },
  tooltip: {
    shared: true,
    useHTML: true,
    shadow: false,
    borderWidth: 0,
    backgroundColor: 'rgba(0,0,0,.7)',
    borderRadius: 8,
    style: {
      color: '#ffffff',
    },
  },
  series: []
};

/* 设置保留几位小数及后缀
  @parmas
  fiex : 保留几位小数，默认是整数
  symbol : 后缀，默认没有后缀
*/
function setValueType({ fiex = 0, symbol = '' }) {
  return {
    toolTip: {
      headerFormat: '<span>{point.key}</span><table>',
      pointFormat: `<tr><td style="padding:0">{series.name}：<b>{point.y:.${fiex}f} ${symbol}</b></td></tr>`,
      footerFormat: '</table>'
    },
    label: {
      format: `{value} ${symbol}`
    }
  }
}

/*
  @parmas
  type :  获取的图表类型
  valueType : {fiex: 0, symbol: ''} 保留几位小数及后缀
  fn : 用于最终组装的回调函数
 */
const getChart = (type, valueType, fn) => {

  // 设置小数位数及后缀
  let { toolTip, label } = setValueType(valueType)

  // 克隆基本图表配置
  let tempBase = _base.deepClone()

  // 将包装好的类型注入到图表插件中
  tempBase.tooltip = {
    ...tempBase.tooltip,
    ...toolTip
  }
  tempBase.yAxis = {
    ...tempBase.yAxis,
    labels: label
  }

  // 初始图表对象
  let obj = {
    ...tempBase,
    chart: {
      type
    }
  }
  return fn(obj)
}

export default getChart
