/**
 * pages模版快速生成脚本,执行命令 npm run lib `文件名`
 */

const fs = require('fs');
const paht =require("path");

const dirName = process.argv[2];

if (!dirName) {
  console.log('组件名称不能为空！');
  console.log('示例：npm run lib test');
  process.exit(0);
}

// 写入shell
const setShell = () => {
  // shell文件路径
  let shellPath = paht.join(__dirname, "lib/pages/Shell.js");
  // 读取shell文件
  let data = fs.readFileSync(shellPath,'utf-8');
  // 将shell文件转为字符串
  let shellStr = data.toString();
  // 添加新模块
  shellStr = shellStr.replace(/\/\/ add/g, (reg) => {
    let temp = `${reg}
  {
    name: '${titleCase(dirName)}',
    url: '/${dirName}'
  },`
    return temp;
  })
  fs.writeFileSync(shellPath, shellStr);
}

//改写路由
const setRouter = () => {
  // 配置路由路径
  let routerPath = paht.join(__dirname, "lib/router.js");
  // 读取路由文件
  let data = fs.readFileSync(routerPath,'utf-8');
  // 将路由转换为字符串
  let routerStr = data.toString();
  // 引入新增的模块
  routerStr = routerStr.replace(/\/\/ add/g, (reg) => {
    let temp = `// ${titleCase(dirName)}
import ${titleCase(dirName)} from './pages/${dirName}';
${reg}`
    return temp
  })
  // 写入路由配置中
  routerStr = routerStr.replace(/\{\/\*路由配置\*\/\}/g, (reg) => {
    let temp = `${reg}
      <Route path="/${dirName}" exact={true} component={${titleCase(dirName)}}/>`
    return temp
  })
  // 写入
  fs.writeFileSync(routerPath, routerStr);
}

// 组件模版
const componentTep = `import React, {Component} from 'react';
import styles from './style.scss';
import PropTypes from 'prop-types';

class ${titleCase(dirName)} extends Component {
  state = {}

  render(){
    return (
      <div>${titleCase(dirName)}</div>
    )
  }
}
export default ${titleCase(dirName)};
`

// 组件scss模版
const componentScssTep = `@import "~_sty/var/index";`;


// 页面模版
const pageTep = `import ${titleCase(dirName)} from '_c/${dirName}'
import styles from './style.scss';

const ${titleCase(dirName)}s = () => {
  return <${titleCase(dirName)} />
}

export default ${titleCase(dirName)}s;
`;

// 页面scss模版
const pageScssTep = `@import "~_sty/var/index";`;

// 创建组件模板
fs.mkdirSync(`./lib/components/${dirName}`);
process.chdir(`./lib/components/${dirName}`);
fs.writeFileSync('index.js', componentTep);
fs.writeFileSync('style.scss', componentScssTep);

fs.mkdirSync(`../../pages/${dirName}`);
process.chdir(`../../pages/${dirName}`);
fs.writeFileSync('index.js', pageTep);
fs.writeFileSync('style.scss', pageScssTep);

// 更新路由和shell
setRouter()
setShell()

console.log('新增成功啦！');

function titleCase(str) {
  const array = str.toLowerCase().split(' ');
  for (let i = 0; i < array.length; i++) {
    array[i] = array[i][0].toUpperCase() + array[i].substring(1, array[i].length);
  }
  const string = array.join(' ');
  return string;
}

process.exit(0);
