const path = require('path');
const webpack = require('webpack');
// const ParallelUglifyPlugin = require('webpack-parallel-uglify-plugin');
const pkg = require('./package.json');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
var banner = `
  copyright (c) 2018-?, vi ${pkg.version}
  description: Autohome chezhiyun
  build time: ${new Date()}
`;

module.exports = {
  entry: './lib/vi.js',
  plugins: [
    // 大抱前清理dist文件夹
    // new CleanWebpackPlugin(['dist']),

    // 预加载react 可以在全局不需要引入React
    new webpack.ProvidePlugin({
      React: 'react'
    }),
    new webpack.BannerPlugin({
      banner: banner
    }),
    new BundleAnalyzerPlugin({
      //  可以是`server`，`static`或`disabled`。
      //  在`server`模式下，分析器将启动HTTP服务器来显示软件包报告。
      //  在“静态”模式下，会生成带有报告的单个HTML文件。
      //  在`disabled`模式下，你可以使用这个插件来将`generateStatsFile`设置为`true`来生成Webpack Stats JSON文件。
      analyzerMode: 'server',
      //  将在“服务器”模式下使用的主机启动HTTP服务器。
      analyzerHost: '127.0.0.1',
      //  将在“服务器”模式下使用的端口启动HTTP服务器。
      analyzerPort: 8881,
      //  路径捆绑，将在`static`模式下生成的报告文件。
      //  相对于捆绑输出目录。
      reportFilename: 'report.html',
      //  模块大小默认显示在报告中。
      //  应该是`stat`，`parsed`或者`gzip`中的一个。
      //  有关更多信息，请参见“定义”一节。
      defaultSizes: 'parsed',
      //  在默认浏览器中自动打开报告
      openAnalyzer: true,
      //  如果为true，则Webpack Stats JSON文件将在bundle输出目录中生成
      generateStatsFile: false,
      //  如果`generateStatsFile`为`true`，将会生成Webpack Stats JSON文件的名字。
      //  相对于捆绑输出目录。
      statsFilename: 'stats.json',
      //  stats.toJson（）方法的选项。
      //  例如，您可以使用`source：false`选项排除统计文件中模块的来源。
      //  在这里查看更多选项：https：  //github.com/webpack/webpack/blob/webpack-1/lib/Stats.js#L21
      statsOptions: null,
      logLevel: 'info'
    }),
    // 打包后压缩JS文件
    // new ParallelUglifyPlugin({
    //   cacheDir: '.cache/',
    //   uglifyJS: {
    //     output: {
    //       comments: false
    //     },
    //     compress: {
    //       warnings: false
    //     }
    //   }
    // })
  ],
  resolve: {
    alias: {
      '_c': path.resolve(__dirname, './lib/components'),
      '_p': path.resolve(__dirname, './lib/pages'),
      '_t': path.resolve(__dirname, './lib/tool'),
      '_sty': path.resolve(__dirname, './style')
    }
  },
  mode: 'production',
  output: {
    filename: 'vi.js',
    path: path.resolve(__dirname, "./dist"),
    libraryTarget: 'umd',
    umdNamedDefine: true
  },
  externals: {
    'auto-gjh': 'auto-gjh',
    'react': {
      root: 'React',
      commonjs: 'react',
      commonjs2: 'react',
      amd: 'react'
    },
    'react-dom': 'react-dom',
    'highcharts': 'highcharts',
    'lodash': 'lodash',
    'moment': 'moment',
    'antd': 'antd',
    '3d-a': '3d-a',
    'html2canvas': 'html2canvas',
    'canvg': 'canvg',
  },
  module: {
    rules: [{
      test: /\.(scss|css)$/,
      use: [
        'style-loader', {
          loader: 'css-loader?sourceMap',
          options: {
            module: true, // 开启css模块化
            localIdentName: '[local]_vi'
          }
        },
        'sass-loader?sourceMap',
        {
          loader: 'sass-resources-loader',
          options: {
            // sourceMap: true,
            resources: [
              path.resolve(__dirname, '.scss'),
            ]
          }
        },
      ],
      include: [path.resolve(__dirname, './lib/components')]
    }, {
      test: /\.(scss|css)$/,
      use: [
        'style-loader', 'css-loader?sourceMap', 'sass-loader?sourceMap'
      ],

      exclude: [ //排除这两个文件夹下面的css文件
        path.resolve(__dirname, './lib/components')
      ]
    }, {
      test: /\.(js|jsx?)$/,
      loader: 'babel-loader',
      // 打包除这个文件之外的文件
      exclude: /node_modules/,
      //打包包括的文件
      include: path.resolve(__dirname, "./lib"),
      //babel打包配置
      // 打包包括的文件
      query: {
        presets: [
          'react', 'stage-0', 'env'
        ],
        plugins: ['transform-runtime', 'transform-class-properties']
      }
    }, {
      test: /\.(png|svg|jpg|gif)$/,
      use: ['file-loader']
    }]
  }
}
