const path = require('path');
const webpack = require('webpack');
// const ParallelUglifyPlugin = require('webpack-parallel-uglify-plugin');
const pkg = require('./package.json');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
// var packCSS = new ExtractTextPlugin('css/style.css');
var banner = `
  copyright (c) 2018-?, vi ${pkg.version}
  description: Autohome chezhiyun
  build time: ${new Date()}
`;

module.exports = {
  entry: './lib/index.js',
  plugins: [
    // 打包前清理dist文件夹
    // new CleanWebpackPlugin(['dist']),

    // 预加载react 可以在全局不需要引入React
    new webpack.ProvidePlugin({
      React: 'react'
    }),
    new webpack.BannerPlugin({
      banner: banner
    }),
    new HtmlWebpackPlugin(
      {
        template: './lib/index.html', //指定要打包的html路径和文件名
        filename: './index.html' //指定输出路径和文件名
      }
    ),
    new MiniCssExtractPlugin({
      // Options similar to the same options in webpackOptions.output
      // all options are optional
      filename: '../pages/style.css',
    }),
    // 打包后压缩JS文件
    // new ParallelUglifyPlugin({
    //   cacheDir: '.cache/',
    //   uglifyJS: {
    //     output: {
    //       comments: false
    //     },
    //     compress: {
    //       warnings: false
    //     }
    //   }
    // })
  ],
  resolve: {
    alias: {
      '_c': path.resolve(__dirname, './lib/components'),
      '_p': path.resolve(__dirname, './lib/pages'),
      '_t': path.resolve(__dirname, './lib/tool'),
      '_sty': path.resolve(__dirname, './style')
    }
  },
  mode: 'production',
  output: {
    path: path.resolve(__dirname, "./pages"),
    filename: 'index.js',
    // libraryTarget: 'umd',
    // umdNamedDefine: true
  },
  module: {
    rules: [{
      test: /\.(scss|css|less)$/,
      use: [
        {
          loader: MiniCssExtractPlugin.loader,
          options: {
            // 这里可以指定一个 publicPath
            // 默认使用 webpackOptions.output中的publicPath
            publicPath: '../'
          },
        },
        'css-loader',
      ],
      use: [
        'style-loader', {
          loader: 'css-loader?sourceMap',
          options: {
            module: true, // 开启css模块化
            localIdentName: '[local]_vi'
          }
        },
        'sass-loader?sourceMap',
        {
          loader: 'sass-resources-loader',
          options: {
            // sourceMap: true,
            resources: [
              path.resolve(__dirname, '.scss'),
            ]
          }
        },
      ]
    }, {
      test: /\.(js|jsx?)$/,
      loader: 'babel-loader',
      // 打包除这个文件之外的文件
      exclude: /node_modules/,
      //打包包括的文件
      include: path.resolve(__dirname, "./lib"),
      //babel打包配置
      // 打包包括的文件
      query: {
        presets: [
          'react', 'stage-0', 'env'
        ],
        plugins: ['transform-runtime', 'transform-class-properties']
      }
    }, {
      test: /\.(png|svg|jpg|gif)$/,
      use: ['file-loader']
    }, {
      test: /\.(woff|woff2|eot|ttf|otf)$/,
      use: ['file-loader']
    },
    {
      test: /\.md$/,
      use: ['html-loader', 'markdown-loader']
    }
    ]
  }
}
