const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
// const CleanWebpackPlugin = require('clean-webpack-plugin');
const webpack = require('webpack');
const fs = require('fs');
const request = require('request');
const HappyPack = require('happypack');
const os = require('os');
const happyThreadPool = HappyPack.ThreadPool({ size: os.cpus().length });
// 同步全局Scss
const fetchScss = () => {
  const url =
    'http://s.autoimg.cn/sou/pm/assets/PaBaLa/css/CZY_Theme/default.scss?t=' +
    Math.random();

  request(url, function (error, response, body) {
    fs.writeFile('style/var.scss', body, err => {
      if (err) {
        throw err;
      } else {
        console.log('同步SCSS完成！');
      }
    });
  });
};

fetchScss();


module.exports = {
  entry: [
    "babel-polyfill", './lib/index.js'
  ],
  plugins: [
    // 预加载react 可以在全局不需要引入React
    new webpack.ProvidePlugin({ React: 'react' }),
    // 静态页面添加js和标题
    new HtmlWebpackPlugin({ title: '汽车之家VI系统', template: './lib/index.html' }),
    // 热加载模块
    new webpack.NamedModulesPlugin(),
    new webpack.HotModuleReplacementPlugin(),
    new HappyPack({
      id: 'happyBabel',
      use: [
        {
          loader: 'babel-loader',
          options: {
            presets: [
              'react', 'stage-0', 'env'
            ],
            plugins: ['transform-runtime', 'transform-class-properties']
          }
        }
      ],
      //共享进程池
      threadPool: happyThreadPool,
      //允许 HappyPack 输出日志
      verbose: true,
    }),
  ],
  resolve: {
    alias: {
      '_c': path.resolve(__dirname, './lib/components'),
      '_p': path.resolve(__dirname, './lib/pages'),
      '_t': path.resolve(__dirname, './lib/tool'),
      '_sty': path.resolve(__dirname, './style')
    }
  },
  devtool: 'inline-source-map',
  mode: 'development',
  output: {
    filename: 'bundle.js',
    path: path.resolve(__dirname, 'dist')
  },
  module: {
    rules: [
      {
        test: /\.(scss|css)$/,
        use: [
          'style-loader',
          {
            loader: 'css-loader?sourceMap',
            options: {
              module: true, // 开启css模块化
              localIdentName: '[local]__vi'
            }
          },
          'sass-loader?sourceMap',
          { loader: 'postcss-loader' },
          {
            loader: 'sass-resources-loader',
            options: {
              // sourceMap: true,
              resources: [
                path.resolve(__dirname, '.scss'),
              ]
            }
          },
        ],
        include: [
          path.resolve(__dirname, './lib/components'),
          path.resolve(__dirname, './lib/pages')
        ]
      }, {
        test: /\.(scss|css)$/,
        use: [
          'style-loader',
          'css-loader?sourceMap',
          'sass-loader?sourceMap',
        ],
        exclude: [
          // 排除这两个文件夹下面的css文件
          path.resolve(__dirname, './lib/components'),
          path.resolve(__dirname, './lib/pages')
        ]
      }, {
        test: /\.(js|jsx?)$/,
        // 打包除这个文件之外的文件
        exclude: /node_modules/,
        // 打包包括的文件
        include: path.resolve(__dirname, './lib'),
        // babel打包配置
        use: [
          {
            loader: 'happypack/loader',
            options: {
              id: 'happyBabel'
            }
          }
        ]
      }, {
        test: /\.(png|svg|jpg|gif)$/,
        use: ['file-loader']
      }, {
        test: /\.(woff|woff2|eot|ttf|otf)$/,
        use: ['file-loader']
      },
      {
        test: /\.md$/,
        use: [
          {
            loader: "html-loader"
          },
          {
            loader: "markdown-loader",
            options: {
              /* your options here */
            }
          }
        ]
      }
    ]
  }
};

// const path = require('path');
// const HtmlWebpackPlugin = require('html-webpack-plugin');
// // const CleanWebpackPlugin = require('clean-webpack-plugin');
// const webpack = require('webpack');
// const fs = require('fs');
// const request = require('request');
// const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
// // 同步全局Scss
// const fetchScss = () => {
//   const url =
//     'http://s.autoimg.cn/sou/pm/assets/PaBaLa/css/CZY_Theme/default.scss?t=' +
//     Math.random();

//   request(url, function (error, response, body) {
//     fs.writeFile('style/var.scss', body, err => {
//       if (err) {
//         throw err;
//       } else {
//         console.log('同步SCSS完成！');
//       }
//     });
//   });
// };

// fetchScss();


// module.exports = {
//   entry: [
//     "babel-polyfill", './lib/index.js'
//   ],
//   plugins: [
//     // 预加载react 可以在全局不需要引入React
//     new webpack.ProvidePlugin({ React: 'react' }),
//     // 静态页面添加js和标题
//     new HtmlWebpackPlugin({ title: '汽车之家VI系统', template: './lib/index.html' }),
//     // 热加载模块
//     new webpack.NamedModulesPlugin(),
//     new webpack.HotModuleReplacementPlugin(),
//     //可视化分析
//     // new BundleAnalyzerPlugin({
//     //   //  可以是`server`，`static`或`disabled`。
//     //   //  在`server`模式下，分析器将启动HTTP服务器来显示软件包报告。
//     //   //  在“静态”模式下，会生成带有报告的单个HTML文件。
//     //   //  在`disabled`模式下，你可以使用这个插件来将`generateStatsFile`设置为`true`来生成Webpack Stats JSON文件。
//     //   analyzerMode: 'server',
//     //   //  将在“服务器”模式下使用的主机启动HTTP服务器。
//     //   analyzerHost: '127.0.0.1',
//     //   //  将在“服务器”模式下使用的端口启动HTTP服务器。
//     //   analyzerPort: 8888,
//     //   //  路径捆绑，将在`static`模式下生成的报告文件。
//     //   //  相对于捆绑输出目录。
//     //   reportFilename: 'report.html',
//     //   //  模块大小默认显示在报告中。
//     //   //  应该是`stat`，`parsed`或者`gzip`中的一个。
//     //   //  有关更多信息，请参见“定义”一节。
//     //   defaultSizes: 'parsed',
//     //   //  在默认浏览器中自动打开报告
//     //   openAnalyzer: true,
//     //   //  如果为true，则Webpack Stats JSON文件将在bundle输出目录中生成
//     //   generateStatsFile: false,
//     //   //  如果`generateStatsFile`为`true`，将会生成Webpack Stats JSON文件的名字。
//     //   //  相对于捆绑输出目录。
//     //   statsFilename: 'stats.json',
//     //   //  stats.toJson（）方法的选项。
//     //   //  例如，您可以使用`source：false`选项排除统计文件中模块的来源。
//     //   //  在这里查看更多选项：https：  //github.com/webpack/webpack/blob/webpack-1/lib/Stats.js#L21
//     //   statsOptions: null,
//     //   logLevel: 'info'
//     // })
//   ],
//   externals: {
//     'auto-gjh': 'auto-gjh',
//     'react': 'react',
//     'highcharts': 'highcharts',
//     'lodash': 'lodash',
//     'moment': 'moment',
//     'antd': 'antd',
//     '3d-a': '3d-a',
//   },
//   resolve: {
//     alias: {
//       '_c': path.resolve(__dirname, './lib/components'),
//       '_p': path.resolve(__dirname, './lib/pages'),
//       '_t': path.resolve(__dirname, './lib/tool'),
//       '_sty': path.resolve(__dirname, './style')
//     }
//   },
//   devtool: 'inline-source-map',
//   mode: 'development',
//   output: {
//     filename: 'bundle.js',
//     path: path.resolve(__dirname, 'dist')
//   },
//   module: {
//     rules: [
//       {
//         test: /\.(scss|css)$/,
//         use: [
//           'style-loader',
//           {
//             loader: 'css-loader?sourceMap',
//             options: {
//               module: true, // 开启css模块化
//               localIdentName: '[local]__vi'
//             }
//           },
//           'sass-loader?sourceMap',
//           { loader: 'postcss-loader' },
//           {
//             loader: 'sass-resources-loader',
//             options: {
//               // sourceMap: true,
//               resources: [
//                 path.resolve(__dirname, '.scss'),
//               ]
//             }
//           },
//         ],
//         include: [
//           path.resolve(__dirname, './lib/components'),
//           path.resolve(__dirname, './lib/pages')
//         ]
//       }, {
//         test: /\.(scss|css)$/,
//         use: [
//           'style-loader',
//           'css-loader?sourceMap',
//           'sass-loader?sourceMap',
//         ],
//         exclude: [
//           // 排除这两个文件夹下面的css文件
//           path.resolve(__dirname, './lib/components'),
//           path.resolve(__dirname, './lib/pages')
//         ]
//       }, {
//         test: /\.(js|jsx?)$/,
//         // 打包除这个文件之外的文件
//         exclude: /node_modules/,
//         // 打包包括的文件
//         include: path.resolve(__dirname, './lib'),
//         // babel打包配置
//         use: [
//           {
//             loader: 'babel-loader',
//             query: {
//               presets: [
//                 'react', 'stage-0', 'env'
//               ],
//               plugins: ['transform-runtime', 'transform-class-properties']
//             }
//           }
//         ]
//       }, {
//         test: /\.(png|svg|jpg|gif)$/,
//         use: ['file-loader']
//       }, {
//         test: /\.(woff|woff2|eot|ttf|otf)$/,
//         use: ['file-loader']
//       },
//       {
//         test: /\.md$/,
//         use: [
//           {
//             loader: "html-loader"
//           },
//           {
//             loader: "markdown-loader",
//             options: {
//               /* your options here */
//             }
//           }
//         ]
//       }
//     ]
//   },

// };
